//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Project: Emergency Call
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/rtlinux/examples/emergencycall/emergencycall.c,v 1.1 2003/01/23 09:28:10 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <rtl.h>
#include <time.h>
#include <pthread.h>
#include <rtl_sched.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define ELLF_MAX_THREADS	2
#define ELLF_SCHEDULER		"yasa_llf"
typedef struct
{
	pthread_t	thread;				// Ptr to thread struct, filled during pthread_create
	char		*name;				// name of thread
	hrtime_t	starttime;			// starttime of first periodically thread
	hrtime_t	period;				// period of thread
	hrtime_t	deadline;			// deadline of thread
	hrtime_t	computationtime;	// time to compute
	int			count;
} ELLF_THREADS;
// definitions for the threads, time in ns
ELLF_THREADS threads[(ELLF_MAX_THREADS+4)*2]=
{
// thread	, name		, starttime	, period	, deadline	, computetime	, count
	{ 0		, "count1"	, 100000000, 3000000000	, 300000000	, 60000000		, 0},		// 500 000 000nsec=0.5ms
	{ 0		, "emcount1", 100000000, 3000000000	, 300000000	, 6000000		, 0},		// 500 000 000nsec=0.5ms
	{ 0		, "count2"	, 100000000, 3000000000	, 300000000	, 60000000		, 0},
	{ 0		, "emcount2", 100000000, 3000000000	, 300000000	, 6000000		, 0},
	{ 0		, "count3"	, 100000000, 3000000000	, 300000000	, 60000000		, 0},
	{ 0		, "emcount3", 100000000, 3000000000	, 300000000	, 6000000		, 0},
	{ 0		, "count4"	, 100000000, 3000000000	, 300000000	, 60000000		, 0},
	{ 0		, "emcount4", 100000000, 3000000000	, 300000000	, 6000000		, 0},
	{ 0		, "count5"	, 100000000, 3000000000	, 300000000	, 60000000		, 0},
	{ 0		, "emcount5", 100000000, 3000000000	, 300000000	, 6000000		, 0},
};
hrtime_t	globalstarttime=0;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Input: void*	parameter is not a pointer. In this case it is the number of the thread
void* emergencytask(void *parameter)
{
	int			threadnumber;
	threadnumber=(int)parameter;										// get current thread number

	do
	{
		// suspend thread, it will be waked up
		// automatically after a deadline is missed
		pthread_suspend_np( pthread_self() );

		// do something important now !
		printk("EMCALL: thread %s missed its deadline.\n", threads[threadnumber+1].name);
	} while(1);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Input: void*	parameter is not a pointer. In this case it is the number of the thread
void* rttask(void *parameter)
{
	hrtime_t	threadstarttime;
	int			threadnumber;
	int			a;
	int			i,policy;
	struct		sched_param sched_param;

	if ( !globalstarttime )												// set starttime of all threads
	{
		globalstarttime=clock_gethrtime(rtl_getschedclock());
	}
	threadnumber=(int)parameter;										// get current thread number
	threads[threadnumber].count++;
	threadstarttime=globalstarttime+threads[threadnumber].starttime;	// add starttime of specified thread

	// Emergency task can only be called if a thread is periodic

	// make periodic now
	pthread_getschedparam( pthread_self(), &policy, &sched_param); 		// fill with default values
	sched_param.yasa_deadline			= threads[threadnumber].deadline;
	sched_param.yasa_period  			= threads[threadnumber].period;
	sched_param.yasa_resumetime			= threadstarttime;
	sched_param.yasa_requiredtime		= threads[threadnumber].computationtime;
	pthread_setschedparam( pthread_self(), policy, &sched_param); 		// fill with default values


	/*
	pthread_make_periodicdeadline_np(									// make thread periodic
							pthread_self(),								// ptr to thread
							threadstarttime,							// set start time
							threads[threadnumber].period,				// set period
							threads[threadnumber].deadline,				// set deadline
							threads[threadnumber].computetime);			// set compute time
	*/
	do
	{
		pthread_wait_np();												// wait until thread starts
		threads[threadnumber].count++;

		rtl_printf("EMCA: thread: %s, %d. period entered!\n", threads[threadnumber].name, threads[threadnumber].count );
		/*
		// do something with time thread[threadnumber].computetime
		for ( i=0; i<200000; i++)
		{
			rtl_
			threadstarttime=clock_gethrtime(rtl_getschedclock());
			a=4;
			a=a>>3;
			a=a*a;
			a=a/(threads[threadnumber].count+1);

			a=4;
			a=a>>3;
			a=a*a;
			a=a/(threads[threadnumber].count+1);

			a=4;
			a=a>>3;
			a=a*a;
			a=a/(threads[threadnumber].count+1);
		}
		*/
		//printk("EMCA: count: %d ready!\n", threadnumber+1);
		if ( threads[threadnumber].count==7 )
			break;


		//printk("EMCA: Thread: %d, count: %d, time: %ld\n", threadnumber+1, threads[threadnumber].count, threadstarttime-globalstarttime);
	} while (1);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int init_module(void)
{
	int					retwert;
	int					i;
	pthread_attr_t		attr;
	struct	sched_param sched_param;

	// prepare initialisation of threads
	pthread_attr_init(&attr);							// construction of attr- and attr.sched object (set default values)
	pthread_attr_setcpu_np(&attr, 0);					// select CPU 0
	pthread_attr_getschedparam( &attr, &sched_param); 	// fill with default values
	sched_param.sched_priority=1;						// set priority to 1 (linux has 0)
	pthread_attr_setschedparam(&attr, &sched_param);	// set scheduling parameter (priority)
	printk("EMCA: Preparing initialisation.\n");

	if ( ! ( yasa_setscheduler( ELLF_SCHEDULER, 0, 0) ) )
	{
		printk("EMCA: Scheduler %s will be used!\n",ELLF_SCHEDULER);
		// create all threads
		for ( i=0; i<ELLF_MAX_THREADS*2; i++)
		{
			pthread_attr_setname(&attr, threads[i].name);	// set name
			pthread_attr_setid(&attr, i+1);					// set id
			retwert=pthread_create(&(threads[i].thread), &attr, rttask, (void*)i );
			if ( retwert )
			{
				printk("EMCA: Thread %d failed to create, reason: %d!\n", i+1, retwert);
			}
		}

		printk("EMCA: Init_Module finished.\n");
	}
	else
	{
		printk("EMCA: Could not init scheduler: %s\n",ELLF_SCHEDULER);
		return 1;
	}

	return 0;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void cleanup_module(void)
{
	int		i;

	printk("EMCA: Cleanup_Module called.\n");
	for ( i=0; i<ELLF_MAX_THREADS*2; i++)
	{
		if ( threads[i].thread )
			pthread_cancel( threads[i].thread);
		threads[i].thread=0;
		printk("EMCA: Thread: %s, count: %d\n", threads[i].name, threads[i].count);
	}
	yasa_setscheduler(YASA_DEFAULT_SCHEDULER, 0, 0);
}


