//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA rtlinux executive declarations
//
// Project:	Yasa 2
// Author :	Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/rtlinux/kernel/include/yasa_rtlinux_functions.h,v 1.2 2003/01/22 17:02:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_RTLINUX_FUNCTIONS_INCLUDE
#define	YASA_RTLINUX_FUNCTIONS_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// function declarations
extern inline int yasa_pthread_setschedparam(YASA_THREAD *thread);
extern INLINE void yasa_setcurrentactivationtime( YASA_THREAD *thread );
extern void yasa_logentry(YASA_THREAD *thread, char type, const YASA_TIME *time, const void *param);



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fill the sched_param structure with default values for non-periodic threads
extern inline void yasa_pthread_sched_init(YASA_THREAD *thread, YASA_SCHEDPARAM *sp)
{
	// only for non-periodic tasks
	sp->sched_priority 			= sched_get_priority_min(0);
	sp->yasa_deadline			= YASA_DEFAULT_DEADLINE;
	sp->yasa_resumetime			= YASA_DEFAULT_RESUMETIME;
	sp->yasa_period				= 0;
	sp->yasa_requiredtime		= YASA_DEFAULT_REQUIREDTIME;
	sp->yasa_dl_behaviour		= YASA_DL_SKIPOVER;
	sp->deadlinetolerance		= 0;
	sp->yasa_dl_emergencythread	= 0;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern inline int yasa_pthread_getschedparam(YASA_THREAD *thread, int *policy,
		    YASA_SCHEDPARAM *sp)
{
	sp->yasa_resumetime		= thread->resume_time;
	sp->yasa_period			= thread->period;
	//*policy=SCHED_FIFO;
	return 0;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set task name
extern inline void yasa_pthread_attr_init(pthread_attr_t *attr)
{
	yasa_pthread_sched_init( pthread_self(), &(attr->sched_param) );
	yasa_id_init( &(attr->id) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set task name
extern inline void pthread_attr_setname_np(pthread_attr_t *attr, const char *name)
{
	yasa_id_set_name( &attr->id, name);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set task id
extern inline void pthread_attr_setid_np(pthread_attr_t *attr, int id)
{
	yasa_id_set_id( &attr->id, id);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// additional function used in yasa
//struct yasa_scheduler_attr;
struct yasa_scheduling_attr;
extern inline void yasa_initialize(YASA_SCHEDULER *sched, int cpu);
extern inline void yasa_cleanup(YASA_SCHEDULER *sched);
extern int inline yasa_rtl_schedule(void);
extern void yasa_pthread_make_periodic_np(YASA_THREAD *thread, YASA_TIME starttime, YASA_TIME period);
int yasa_setscheduler(const struct yasa_scheduling_attr *schedulingattr, int cpu_id);
extern void yasa_pthread_create(YASA_THREAD *thread, const YASA_ATTRIBUTES *attr);
extern void yasa_pthread_exit(void *retval);



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																// ifndef YASA_RTLINUX_FUNCTIONS_INCLUDE