//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA original rtlinux-scheduler
//
// Project:	Yasa 2
// Author :	Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/rtlinux/kernel/include/yasa_rtlinux_sched.h,v 1.2 2003/01/22 17:02:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_RTLINUX_SCHED_INCLUDE
#define	YASA_RTLINUX_SCHED_INCLUDE

#include <yasa/yasa.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_THREAD* ortl_schedule(YASA_SCHEDULER *sched, const YASA_TIME *now, YASA_TIME *ptime)
{
	struct rtl_thread_struct *t;
	struct rtl_thread_struct *preemptor;
	struct rtl_thread_struct *new_task=0;

	for (t = sched->rtl_tasks; t; t = t->next) {
		/* expire timers */

		if (test_bit(RTL_THREAD_TIMERARMED, &t->threadflags)) {
			if (*now >= t->resume_time) {
				clear_bit(RTL_THREAD_TIMERARMED, &t->threadflags);
				rtl_sigaddset (&t->pending, RTL_SIGNAL_TIMER);
				if (t->period != 0) { /* periodic */
					YASAL_LOG_THREAD_ACTIVATE(t, now, &YASA_RESUME_TIME(t) );
					t->resume_time += t->period;
					/* timer overrun */
#ifndef CONFIG_RTL_OLD_TIMER_BEHAVIOUR
					while (*now >= t->resume_time) {
						YASAL_LOG_THREAD_SKIPPED(t, &YASA_RESUME_TIME(t), &YASA_RESUME_TIME(t) );
						t->resume_time += t->period;
/* 						rtl_printf("overrun"); */
					}
#endif
				} else {
					t->resume_time = HRTIME_INFINITY;
				}
			}
		}

		/* and find highest priority runnable task */
		if ((t->pending & ~t->blocked) && (!new_task ||
			(t->sched_param.sched_priority > new_task->sched_param.sched_priority))) {
				new_task = t;
		}
	}

	if ( YASA_CALCPREEMPTIONTIME(sched) && new_task && ( preemptor=find_preemptor(sched, new_task) ) )
	//if ( new_task && ( preemptor=find_preemptor(sched, new_task) ) )
	{
		*ptime= preemptor->resume_time;
	}
	else
	{
		// if linux thread is be chosen later in scheduler, the preemption will be
		// corrected
		//*ptime = *now + (HRTICKS_PER_SEC / HZ) / 2;				// OS tick
		*ptime = HRTIME_INFINITY;
	}

	return new_task;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// static definitions
const YASA_SCHEDULER_ATTR ortl_prioattr=
{
	0,					// next
	{ 0, "rtl_prio" },	// id & name
	"1.0",				// scheduler version
	0,					// userdata
	0,					// init()
	0,					// cleanup()
	ortl_schedule,		// schedule()
	0,					// findpreemptor()
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const YASA_SCHEDULING_ATTR ortl_schedulingattr=
{
	// function		, end time			, tick							, calc preemption time
	&ortl_prioattr	, HRTIME_INFINITY	, (HRTICKS_PER_SEC / HZ) / 2	, 1
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif													// ifndef YASA_RTLINUX_SCHED_INCLUDE