//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA rtlinux project startup code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/rtlinux/sources/project_startup/yasa_projectstartup.c,v 1.2 2003/01/23 09:18:30 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// YP_SCHEDULER_ATTR & YP_THREAD_ATTR are normally mapped in $(YASA_PROJECT)/yasa_projecttaskset.h
extern YASA_ENVIRONMENT_ATTR	YP_ENVIRONMENT_ATTR;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct yp_thread_info
{
	YASA_THREAD			*thread;
	YASA_THREAD_ATTR	*attr;
	int					exited;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// internal static variables
struct yp_thread_info 	*threads=0;
int						threadcount;
int						fifohandle;
pthread_mutex_t			*yp_firstmutex=0;
int						yp_mutexcount=0;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// internal functions
void yp_cancel_threads(struct yp_thread_info **threads, int threadcount);
void yp_reset_schedulers(void);
int  yp_alloc_mutexes(YASA_MUTEX **firstmutex, YASA_ENVIRONMENT_ATTR *environmentattr, int *mutexcount);
void yp_free_mutexes(YASA_MUTEX **firstmutex, int *mutexcount);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Input: void*	parameter is not a pointer. In this case it is the number of the thread
void* yp_starttask(void *parameter)
{
	struct yp_thread_info	*tinfo;
	void					*userdata;
	void					*retvalue;
	YASA_TIME				now;

	// get thread attr
	tinfo=(struct yp_thread_info*)parameter;

	now = tinfo->thread->scheduler->clock->gethrtime(tinfo->thread->scheduler->clock);
	YASAL_LOG_MESSAGE( tinfo->thread, &now, "        Start part 1");


	// call create thread
	if ( tinfo->attr->threadinit )
	{
		if ( ! ( retvalue=tinfo->attr->threadinit( &tinfo, &userdata ) ) )
		{
			tinfo->exited=1;
			pthread_exit( retvalue );
		}
	}

	now = tinfo->thread->scheduler->clock->gethrtime(tinfo->thread->scheduler->clock);
	YASAL_LOG_MESSAGE( tinfo->thread, &now, "        Start part 2");

	// call start thread. The macro "YASA_PERIODIC_START" or "YASA_NONPERIODIC_START" used within
	// threadexecute() will suspend the thread
	retvalue=0;
	if ( tinfo->attr->threadexecute )
	{
		retvalue=tinfo->attr->threadexecute( &userdata );
		//rtl_printf("Thread %d execute() finished!\n", yasa_id_get_name(&(pthread_self()->id));
	}

	// call start thread
	if ( tinfo->attr->threadcleanup )
	{
		tinfo->attr->threadcleanup( &userdata );
	}

	tinfo->exited=1;
	return retvalue;
}


	//return (void*)retvalue;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_ERROR yp_set_schedulers(const YASA_SCHEDULING_ATTR *schedulingattr)
{
	YASA_ERROR		retvalue=0;
	YASA_SCHEDULER	*sched;
	int				cpu;
	YASA_TIME		now;
	rtl_irqstate_t 	irqstate;


	//printk("YPL: yp_set_schedulers entered!\n");
	if ( YP_MAX_CPU > NR_CPUS )
	{
		printk("YPE: Error, cpu number to high!\n");
		return YASAE_WRONG_CPU;
	}


	// spinlock !
	rtl_no_interrupts(irqstate);

	now=LOCAL_SCHED->clock->gethrtime(LOCAL_SCHED->clock);

	for ( cpu=0; cpu<YP_MAX_CPU; cpu++ )
	{
		sched					= sched_data(cpu);
		sched->schedulerstart	= now;

		// initialize logging functions
		if ( ! ( retvalue=yasa_initlogging( &(sched->logfile), YP_LOGGINGSIZE ) ) )
		{
			// the schedule functions of each scheduler are linked automatically because they are
			// defined in scheduler_attr
			if ( ! ( retvalue=yasa_setscheduler( schedulingattr, cpu) ) )
			{
				if ( schedulingattr->schedulerattr )
					printk("YPL: CPU: %d, scheduler %s.\n", cpu, yasa_id_get_name(& (schedulingattr->schedulerattr->id) ) );
				else
					printk("YPL: CPU: %d, is using default rtl-scheduler.\n", cpu);
			}
		}
		if ( retvalue )
			break;
		schedulingattr++;
	}

	rtl_restore_interrupts(irqstate);

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yp_reset_schedulers()
{
	int			cpu;

	// reset schedulers
	//printk("YPL: ResetSchedulers entered!\n");
	for ( cpu=0; cpu<YP_MAX_CPU; cpu++ )
	{
		yasa_setscheduler(0, cpu);
		yasa_stoplogging( &sched_data(cpu)->logfile );
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_ERROR yp_create_threads( struct yp_thread_info **threadsptr, YASA_ENVIRONMENT_ATTR *environmentattr, int *threadcount)
{
	YASA_THREAD_ATTR 		*newthreads=environmentattr->firstthreadattr;
	YASA_THREAD_ATTR		*newthread;
	pthread_attr_t			attr;
	struct yp_thread_info	*threads=0;


	//printk("YPL: yp_create_threads entered!\n");

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// count threads and create workspace
	newthread=newthreads;
	*threadcount=0;
	*threadsptr=0;
	while (newthread)
	{
		(*threadcount)++;
		newthread=newthread->next;
	};

	if ( *threadcount==0 )
		return YASAE_ERROR_IN_YASA_ATTRIBUTS;
	threads=YASA_MALLOC( sizeof(*threadcount*sizeof(struct yp_thread_info) ) );
	*threadsptr=threads;
	if ( ! threads )
	{
		printk("YPE: Could not create yasa_thread_info with size: %d!\n", sizeof(*threadcount*sizeof(struct yp_thread_info) ) );
		return YASAE_OUT_OF_MEMORY;
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// create threads with default values now. Later we will set the scheduling policies more in detail
	newthread=newthreads;
	while (newthread)
	{
		// prepare initialisation of threads
		threads->exited=0;
		//threads->returnvalue=0;
		threads->attr=newthread;
		pthread_attr_init(&attr);							// construction of attr- and attr.sched object (set default values)
		if ( newthread->cpu >= NR_CPUS )
		{
			printk("YPE: Error, cpu number of thread %s to high\n", newthread->id.name);
			return YASAE_WRONG_CPU;
		}
		pthread_attr_setcpu_np(&attr, newthread->cpu);		// select CPU 0
		//pthread_attr_getschedparam( &attr, &sched_param); // fill with default values
		//pthread_attr_setschedparam(&attr, &sched_param);	// set scheduling parameter (priority)
		pthread_attr_setname_np(&attr, newthread->id.name);	// set name
		pthread_attr_setid_np(&attr, newthread->id.id);		// set id
		if  ( pthread_create( &(threads->thread), &attr, yp_starttask, threads ) != 0)
			return YASAE_CREATE_THREAD;
		newthread=newthread->next;
		threads++;
	};

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// insert emergency threads
	newthread=newthreads;
	while (newthread)
	{
		if ( newthread->emergencythread )
		{
			printk("Emergencythreads not supported yet!\n");
		}
		newthread=newthread->next;
	}
	//printk("YP: CreateThreads finished!\n");
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_ERROR yp_synchronize(void)
{
	int						i;
	YASA_TIME				now;
	YASA_SCHEDULER			*sched;
	int						cpu;
	rtl_irqstate_t 			irqstate;
	struct	sched_param 	sched_param;
	int						policy;

	//printk("YPL: yp_synchronize entered!\n");
	if ( YP_MAX_CPU > NR_CPUS )
	{
		printk("YPE: Error, cpu number to high!\n");
		return YASAE_WRONG_CPU;
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// - all rt threads must be suspended here. We can use a polling wait because the rtthreads are using a
	//   higher priority and can not be interrupted by this kernel module. So the must finished here
	// - we are the only running task
	// - no interrupts allowed
	// - read clock (clk)
	// - set resume time of each thread and scheduler starttime ( clk+security time(1s) )
	// - interrupts allowed
	// - we do not need to call rtl_schedule because we have a one shot timer loaded with a time
	//   tick. If we call rtl_schedule, we would call it twice maybe
	//	 * once after the timer interrupt is triggered
	//   * once here

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	for ( i=0; i<threadcount; i++ )
	{
		while( ( ! threads[i].exited ) && test_bit( RTL_SIGNAL_READY, &(threads[i].thread->pending) ) )
		{
			//usleep(10);			// sleep will fail -> hanging system
		};
		//printk("thread %d suspended!\n", threads[i].thread->yasa_id);
	}

#define RTLINUX_STARTOFFSET		YP_ENVIRONMENT_ATTR.synchronizationoffset
//#define RTLINUX_STARTOFFSET		50000000
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// all realtime threads are suspended now
	// set/correct resume times
	rtl_no_interrupts(irqstate);
	now=LOCAL_SCHED->clock->gethrtime(LOCAL_SCHED->clock)+RTLINUX_STARTOFFSET;
	for ( i=0; i<threadcount; i++ )
	{
		pthread_getschedparam( threads[i].thread, &policy, &sched_param); 				// fill with default values
		sched_param.sched_priority		=threads[i].attr->priority;
		sched_param.yasa_period			=threads[i].attr->period;
		sched_param.yasa_requiredtime	=threads[i].attr->requiredtime;
		sched_param.yasa_deadline		=threads[i].attr->deadline;
		sched_param.yasa_resumetime		=threads[i].attr->offset+now;
		sched_param.deadlinetolerance	=threads[i].attr->deadlinetolerance;
		sched_param.yasa_dl_emergencythread	= 0;
		//sched_param.yasa_dl_behaviour=threads[i].thread->attr->deadlinebehaviour;
		// set/correct emergency threads
		pthread_setschedparam(threads[i].thread, policy, &sched_param);		// set scheduling parameter (priority)
		/*
		printk("thread %d new schedparam set!\n", threads[i].thread->yasa_id);
		printk("thread %d, prio:%d, pe:%d, dl:%d, rq:%d, rs:%d\n", threads[i].attr->id,
							sched_param.sched_priority, sched_param.yasa_period,
							sched_param.yasa_deadline, sched_param.yasa_requiredtime,
							sched_param.yasa_resumetime );
		*/
		YASAL_LOG_SCHEDULER_SYNCHRONIZED_START( LOCAL_SCHED, threads[i].thread->id.id, &now, 0 );
	}

	//YASAL_LOG_SCHEDULER_MESSAGE( LOCAL_SCHED, &now, "Synchronized start!");
	YASAL_LOG_SCHEDULER_SYNCHRONIZED_START( LOCAL_SCHED, YASA_SCHEDULER_ID, &now, 0 );
	YASAL_LOG_SCHEDULER_SYNCHRONIZED_START( LOCAL_SCHED, YASA_IDLE_ID, &now, 0 );

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// wakeup threads
	for ( i=0; i<threadcount; i++ )
	{
		if ( ( ! threads[i].exited ) && threads[i].thread->resume_time == now )
		{
			RTL_MARK_READY(threads[i].thread);
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// start scheduling
	for ( cpu=0; cpu<YP_MAX_CPU; cpu++ )
	{
		sched=sched_data(cpu);

		// set timers
		rtl_spin_lock (&sched->rtl_tasks_lock);
		(sched->clock)->settimer(sched->clock, 0);
		rtl_spin_unlock (&sched->rtl_tasks_lock);

	#ifdef CONFIG_SMP
		rtl_reschedule (cpu_id);
	#else
		rtl_schedule();
	#endif

	}
	rtl_restore_interrupts(irqstate);

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int yp_alloc_mutexes(YASA_MUTEX **firstmutex, YASA_ENVIRONMENT_ATTR *environmentattr, int *mutexcount)
{
	YASA_MUTEX_ATTR 	*firstattr=environmentattr->firstmutexattr;
	int					retvalue=0;
	pthread_mutexattr_t	mutexattr;
	int					i;
	YASA_MUTEX_ATTR		*yasamutexattr=firstattr;

	printk("YPL: yp_alloc_mutexes entered!\n");

	// args are valid ?
	if ( firstmutex && firstattr && mutexcount )
	{
		// count mutex
		*firstmutex=0;
		*mutexcount=0;
		while ( yasamutexattr )
		{
			(*mutexcount)++;
			yasamutexattr=yasamutexattr->next;
		};

		// alloc mutex memory
		*firstmutex=YASA_MALLOC( sizeof(pthread_mutex_t ) * (*mutexcount) );
		printk("YPL: yp_alloc_mutexes entered2!\n");
		if ( *firstmutex )
		{
			yasamutexattr=firstattr;
			yasa_calculateceilings( environmentattr->firstmutexattr, environmentattr->firstthreadattr);
			printk("YPL: yp_alloc_mutexes entered3!\n");

			for ( i=0;i<*mutexcount && retvalue==0;i++)
			{
				pthread_mutexattr_init( &mutexattr );
				pthread_mutexattr_copyid_np( &mutexattr, &yasamutexattr->id);
				//pthread_mutexattr_setdynamicceiling_np( &mutexattr, yasamutexattr->dynamicceiling);
				pthread_mutexattr_setpreemptceiling_np( &mutexattr, yasamutexattr->preemptceiling);
				pthread_mutexattr_setprioceiling( &mutexattr, yasamutexattr->priorityceiling);
				pthread_mutexattr_setprotocol( &mutexattr, yasamutexattr->protocol );
				printk("YPL: yp_alloc_mutexes entered4!\n");

				retvalue=pthread_mutex_init( &((*firstmutex)[i]), &mutexattr);
				yasamutexattr=yasamutexattr->next;
			}
		}
		if ( retvalue )
			yp_free_mutexes(firstmutex, mutexcount);
	}

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yp_free_mutexes(YASA_MUTEX **firstmutex, int *mutexcount)
{
	int	i;

	printk("YPL: yp_free_mutexes entered!\n");
	if ( firstmutex && *firstmutex )
	{
		for ( i=0; i<*mutexcount; i++)
		{
			printk("mutex: \"%s\" destroyed!\n", yasa_id_get_name(&((*firstmutex)[i]).id ) );
			pthread_mutex_destroy( &((*firstmutex)[i]) );
		};
		YASA_FREE( *firstmutex );
		*firstmutex=0;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
pthread_mutex_t * yp_getmutex(int id)
{
	int	i;

	for ( i=0; i<yp_mutexcount; i++)
	{
		if ( yasa_id_get_id( &(yp_firstmutex)[i].id ) == id )
		{
			return &(yp_firstmutex[i]);
		}
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int init_module(void)
{
	YASA_ERROR			retvalue=YASAE_NO_ERROR;

	//printk("YPL: InitModule entered!\n");
//	retvalue=yp_set_schedulers( (YP_SCHEDULING_ATTR) );
	retvalue=yp_set_schedulers( (environmentattr.firstschedulingattr) );
	if ( ! retvalue )
	{
//		retvalue=yp_alloc_mutexes(&yp_firstmutex, &YP_MUTEX_ATTR, &yp_mutexcount);
		retvalue=yp_alloc_mutexes(&yp_firstmutex, &environmentattr, &yp_mutexcount);
		if ( !retvalue )
		{
//			retvalue=yp_create_threads( &threads, &YP_THREAD_ATTR, &threadcount );
			retvalue=yp_create_threads( &threads, &environmentattr, &threadcount );
			if ( ! retvalue )
			{
				// Synchronize all threads
				yp_synchronize();
				// Return with no error

				return 0;
			}
		}
	}

	// There were errors occured
	yp_free_mutexes(&yp_firstmutex, &yp_mutexcount);
	yp_cancel_threads(&threads, threadcount);
	yp_reset_schedulers();
	printk("YPE: Error %d during initialization!\n", retvalue);
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yp_cancel_threads(struct yp_thread_info **threads, int threadcount)
{
	int					i;
	void				*returnvalue;

	//printk("YPL: CancelThreads entered!\n");
	if ( threads && *threads )
	{
		if ( *threads )
		{
			for (i=0 ; i<threadcount; i++)
			{
				if ( (*threads)[i].thread )
				{
					pthread_cancel( (*threads)[i].thread );
					pthread_join( (*threads)[i].thread, &returnvalue );
				}
				(*threads)[i].thread=0;
				(*threads)[i].attr=0;
			}
			YASA_FREE(*threads);
		}
		*threads=0;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void cleanup_module(void)
{
	//printk("YEL: Cleanup_Module for project \"%s\" called.\n", YP_PROJECTNAME);
	yp_cancel_threads(&threads, threadcount);
	yp_free_mutexes( &yp_firstmutex, &yp_mutexcount );
	yp_reset_schedulers();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int yp_printlogfile(YASA_LOGFILE *logfile, YASA_TIME *schedulerstart, int cpu, void* handle)
{
	return yasa_printlogfile(logfile, schedulerstart, cpu, handle);
}
