//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA scheduling includes
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/include/yasa/yasa_scheduling.h,v 1.2 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_SCHEDULING_INCLUDE
#define	YASA_SCHEDULING_INCLUDE

#include <yasa/yasa.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This structure describes the behaviour of each scheduler
struct yasa_scheduler_attr
{
	struct YASA_SCHEDULER_ATTR*	next;
	YASA_ID						id;
	const char *				scheduler_version;	// scheduler version
	void*						userdata;			// not used yet

	// support functions of scheduler
	int							(*yasa_init)(YASA_SCHEDULER *scheduler);
	void						(*yasa_cleanup)(YASA_SCHEDULER *scheduler);
	YASA_THREAD* 				(*yasa_schedule)(YASA_SCHEDULER *sched, const YASA_TIME *now, YASA_TIME *ptime);
	YASA_THREAD* 				(*yasa_findpreemptor)(YASA_SCHEDULER *sched, YASA_TIME *now);
};
typedef struct yasa_scheduler_attr YASA_SCHEDULER_ATTR;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This structure describes the behaviour of each cpu
struct yasa_scheduling_attr
{
	const struct yasa_scheduler_attr	*schedulerattr;
	YASA_TIME							schedulingend;	// when should the simulation end
	YASA_TIME							tick;			// should be set everytime to a correct value
														// even if a preemption time is calculated
	int									calcpreemptiontime;
};
typedef struct yasa_scheduling_attr YASA_SCHEDULING_ATTR;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This structure describes the global behaviour of the environment
struct yasa_environment_attr
{
	const struct yasa_scheduling_attr	*firstschedulingattr;			// first scheduler attr
	YASA_THREAD_ATTR					*firstthreadattr;				// first thread	attr
	YASA_MUTEX_ATTR						*firstmutexattr;				// first mutex attr
	YASA_TIME							synchronizationoffset;
};
typedef struct yasa_environment_attr YASA_ENVIRONMENT_ATTR;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// After finishing a thread this function can be called to set next activation time.
// The activation time may be in a past!
// This function can be called by Schedulers and Executives!

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// These function can be called by schedulers only !
extern INLINE void yasa_setdeadline( YASA_THREAD *thread );
extern INLINE int yasa_deadlinemissed(YASA_THREAD *thread, const YASA_TIME *now);
extern INLINE void yasa_setnextactivationtime( YASA_THREAD *thread, const YASA_TIME *now);
extern INLINE void yasa_setcurrentactivationtime( YASA_THREAD *thread );




#endif																			// ifndef YASA_SCHEDULING_INCLUDE
