//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA linux scheduler
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/schedulers/linux/linux.c,v 1.3 2003/01/23 09:18:33 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "linux.h"



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//static INLINE int linux_goodness( YASA_THREAD *p)
int linux_goodness( YASA_THREAD *p)
{
	int				weight;
	weight = p->counter;
	if (!weight)
		return weight;

		/* .. and a slight advantage to the current MM */
		if ( p == YASA_CURRENT_THREAD ) //p->mm == this_mm || !p->mm)
			weight += 1;
		//weight += 20 - p->nice;

		//weight += p->priority;
	return weight;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// ptime = preemption time (preloaded with system tick)
YASA_THREAD* linux_schedule(YASA_SCHEDULER *scheduler, const YASA_TIME *now, YASA_TIME *ptime)
{
	YASA_THREAD		*thread;
	YASA_THREAD		*newthread;
	int				reschedule;
	int				weight,c;


	// scheduling cycle
	do
	{
		if ( YASA_CURRENT_THREAD && YASA_SCHEDULABLE( YASA_CURRENT_THREAD ) )
		{
			c = linux_goodness( YASA_CURRENT_THREAD );
			newthread=YASA_CURRENT_THREAD;
		}
		else
		{
			//	c = -1000;
			c = 0;
			newthread=0;
		}

		reschedule=0;
		for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
		{
			if ( YASA_CHECK_NEXT_ACTIVATION(thread) )
				yasa_setnextactivationtime(thread, now);

			// Should the thread be (re)started in future?
			// thread->threadflags&TIMER_ARMED
			// The thread should be restarted. Did we reach the resume time?
			if ( YASA_TEST_TIMER_ARMED(thread) && *now >= YASA_RESUME_TIME(thread) )
			{
				// timer is no more armed. thread is ready to compute
				// thread->threadflags&=~TIMER_ARMED
				YASA_UNSET_TIMER_ARMED(thread);

				// thread can be choosen in this scheduling !
				// thread->pending will 'TRUE'
				YASA_SIGNAL_TIMER(thread);

				// set next(!) resume time
				yasa_setdeadline(thread);			// set deadline of current thread
			}

			// find highest priority runnable task
			if ( YASA_SCHEDULABLE(thread) )
			{
				// if deadline missed, there can be started an emergency thread
				// rescheduling neccessary !!!
				if ( YASA_TEST_DEADLINE_MISSED(thread,now) )
					reschedule|=yasa_deadlinemissed(thread, now);
				else
				{
					weight = linux_goodness(thread);
					if (weight > c)
						c = weight, newthread = thread;
				}
			}
		}

	} while (reschedule);

	if (!c)
	{
		for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
		{
			thread->counter = ( thread->counter >> 1) + YASA_PRIORITY(thread) ;
		}
	}

	/////////////////////////////////////////////////////////////////////////////////////////
	// determine preemption time. Please pay attention to:
	// 1. should the tick or the preemption time be used
	// 2. what happens if newthread=0 ?
	// 3. what happens with non computing threads having deadlines until *ptime ?
	// 4. You cannot be sure to get valid values if the project does not support this scheduler
	//    Example: In priority based schedulers the priority should be analyzed only?
	YASA_SETTICKSCHEDULINGTIME(ptime, now, scheduler)
	//*ptime = *now + scheduler->tick;


	return newthread;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// static definitions
const YASA_SCHEDULER_ATTR linux_attr=
{
	0,						// next
	{ 0, "ys_linux"},		// id & name
	LINUX_VERSION,			// scheduler version
	0,						// userdata
	0,						// init()
	0,						// cleanup()
	linux_schedule,			// schedule()
	0,						// findpreemptor()
};

