//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA resource handling (mutexs etc.) definitions
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/15
// $Header: /sources/yasa/yasa/yasa_resources.c,v 1.2 2003/01/23 09:18:33 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definitions for PCP
// YASA_PCP_STACK				 = This stack holds all threads which could not lock a mutex and were suspended
// YASA_PCP_SYSTEM_CEILING_STACK = thread, that wanted to lock a mutex
YASA_PCP_STACK					PCP_Stack[YASA_PCP_MAXSTACK];
int 							PCP_StackCount=0;
YASA_PCP_SYSTEM_CEILING_STACK	PCP_SystemCeilingStack[YASA_PCP_MAXSTACK]={ {YASA_MIN_PRIORITY-1,0,0} };
int 							PCP_SystemCeilingCount=0;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definitions for Kernelized Monitor
int				KP_Count=0;
YASA_THREAD*	(*KP_ScheduleOld)(YASA_SCHEDULER *scheduler, const YASA_TIME *now, YASA_TIME *ptime);
YASA_THREAD*	KP_Schedule(YASA_SCHEDULER *scheduler, const YASA_TIME *now, YASA_TIME *ptime)
{
	*ptime=YASA_TIME_INFINITY;
	return YASA_CURRENT_THREAD;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// call is denied if mutex is locked !
int yasa_mutex_try_lock(YASA_MUTEX *mutex)
{
	//YASA_THREAD		*thread=YASA_CURRENT_THREAD;
	//YASA_SCHEDULER	*sched=YASA_CURRENT_SCHEDULER;
	switch (mutex->protocol)
	{
		// BPIP=Basic Priority Inheritance Protocol (used in RTLinux PTHREAD_PRIO_INHERIT)
		// PIP=Priority Inheritance Protocol
		case YASASP_PIP:	if ( YASA_MUTEX_OLD_PRIORITY(mutex) == YASA_MUTEX_DEFAULT_PRIORITY)
								YASA_MUTEX_OLD_PRIORITY(mutex) = YASA_PRIORITY( YASA_CURRENT_THREAD );
							else
							{
								YASAL_LOG_MUTEX_PRIORITY_CHANGED( mutex->thread, 0, &YASA_PRIORITY( mutex->thread ), &YASA_PRIORITY( YASA_CURRENT_THREAD ) );
								YASA_PRIORITY( mutex->thread ) = YASA_PRIORITY( YASA_CURRENT_THREAD );
							}
							break;

		// Priority Ceiling Protocol (used in RTLinux _RTL_POSIX_THREAD_PRIO_PROTECT)
		case YASASP_PCP:	if ( YASA_PRIORITY(YASA_CURRENT_THREAD) > YASA_MUTEX_PRIORITY_CEILING(mutex) )
								return EINVAL;
							if ( ( (YASA_PRIORITY(YASA_CURRENT_THREAD) > PCP_SystemCeilingStack[PCP_SystemCeilingCount].ceiling) &&
							       (YASA_MUTEX_OLD_PRIORITY(mutex) == YASA_MUTEX_DEFAULT_PRIORITY) ) ||
								 (PCP_SystemCeilingStack[PCP_SystemCeilingCount].thread==YASA_CURRENT_THREAD ) )
							{
								// lock mutex
								if ( PCP_SystemCeilingCount > YASA_PCP_MAXSTACK )
									return EBUSY;
								YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0, &PCP_SystemCeilingStack[PCP_SystemCeilingCount].ceiling, &YASA_MUTEX_PRIORITY_CEILING(mutex) );
								PCP_SystemCeilingCount++;
								PCP_SystemCeilingStack[PCP_SystemCeilingCount].ceiling=YASA_MUTEX_PRIORITY_CEILING(mutex) ;
								PCP_SystemCeilingStack[PCP_SystemCeilingCount].thread=YASA_CURRENT_THREAD;
								PCP_SystemCeilingStack[PCP_SystemCeilingCount].stackpos=PCP_StackCount;
								YASA_MUTEX_OLD_PRIORITY(mutex) = YASA_PRIORITY( YASA_CURRENT_THREAD );
							}
							else
							{
								int	i,j=0;
								// mutex is locked or cannot be locked
								YASAL_LOG_MUTEX_PRIORITY_CHANGED( PCP_SystemCeilingStack[PCP_SystemCeilingCount].thread, 0, &YASA_PRIORITY( PCP_SystemCeilingStack[PCP_SystemCeilingCount].thread ), &YASA_PRIORITY( YASA_CURRENT_THREAD ) );
								YASA_PRIORITY( PCP_SystemCeilingStack[PCP_SystemCeilingCount].thread ) = YASA_PRIORITY( YASA_CURRENT_THREAD );
								// notice that mutex was blocked through the current one
								for ( i=PCP_SystemCeilingStack[PCP_SystemCeilingCount].stackpos ; i<PCP_StackCount; i++)
								{
									if ( PCP_Stack[i].mutex==mutex )		// mutex found
										j=1;
								}
								// This mutex(2) is not in list. After unlock of the current mutex(1), all threads
								// of the mutex(2) will be waked up.
								if ( !j )
								{
									PCP_Stack[PCP_StackCount].mutex=mutex;
									PCP_StackCount++;
								}
								return EBUSY;
							}
							break;

		// IPCP=Instant Priority Ceiling Protocol
		// CSP=Ceiling Semaphore Protocol (used in RTLinux _RTL_POSIX_THREAD_PRIO_PROTECT)
		// if current priority>ceiling then priority is invalid. Otherwise mutex is not locked because
		// priority is smaller than ceiling
		case YASASP_CSP:	if ( YASA_PRIORITY(YASA_CURRENT_THREAD) > YASA_MUTEX_PRIORITY_CEILING(mutex) )
								return EINVAL;
							YASA_MUTEX_OLD_PRIORITY(mutex) = YASA_PRIORITY( YASA_CURRENT_THREAD );
							YASAL_LOG_MUTEX_PRIORITY_CHANGED( YASA_CURRENT_THREAD, 0, &YASA_PRIORITY( YASA_CURRENT_THREAD ), &YASA_MUTEX_PRIORITY_CEILING(mutex) );
							YASA_PRIORITY( YASA_CURRENT_THREAD ) = YASA_MUTEX_PRIORITY_CEILING(mutex);
							break;



		// Dynamic Priority Ceiling Protocol
#ifdef YP_SUPPORT_SYNCHRONIZATION_DPCP
		case YASASP_DPCP:	if ( yasa_dpcp_equaldynamicpriority ( &mutex->dynamicceiling, &DPCP_MIN_PRIORITY ) )
								yasa_dpcp_copydynamicpriority( YASA_CURRENT_THREAD, &mutex->dynamicceiling );
						//	if ( YASA_DYNPRIORITY(YASA_CURRENT_THREAD) < mutex->dynamicceiling )
							/*if ( cmp_dyn_prio( YASA_CURRENT_THREAD, mutex->dynamicceiling) )
							{
								PRINTK("Error in arguments!\n");
								return EINVAL;
							}*/
							if ( ( yasa_dpcp_cmpdynamicpriority( &(DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].dynamicceiling) ) ||
								   (DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].thread==YASA_CURRENT_THREAD ) ) &&
								   ! YASA_MUTEX_LOCKED(mutex) )
							{
								// lock mutex
								if ( DPCP_SystemCeilingCount > YASA_DPCP_MAXSTACK )
									return EBUSY;
								yasa_logceiling( &DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].dynamicceiling, &mutex->dynamicceiling);
								DPCP_SystemCeilingCount++;
								DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].dynamicceiling=mutex->dynamicceiling ;
								DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].thread=YASA_CURRENT_THREAD;
								DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].stackpos=DPCP_StackCount;
								//YASA_MUTEX_OLD_PRIORITY(mutex) = YASA_PRIORITY( YASA_CURRENT_THREAD );
							}
							else
							{	// mutex is locked or cannot be locked
								int	i,j=0;
								/*
								YASAL_LOG_MUTEX_PRIORITY_CHANGED( DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].thread, 0,
																  &YASA_DYNPRIORITY( DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].thread ),
																  &YASA_DYNPRIORITY( YASA_CURRENT_THREAD ) );
								*/
								//YASA_PRIORITY( DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].thread ) = YASA_PRIORITY( YASA_CURRENT_THREAD );

								// notice that mutex was blocked through the current one
								for ( i=DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].stackpos ; i<DPCP_StackCount; i++)
								{
									if ( DPCP_Stack[i].mutex==mutex )		// mutex found
										j=1;
								}
								// This mutex(2) is not in list. After unlock of the current mutex(1), all threads
								// of the mutex(2) will be waked up.
								if ( !j )
								{
									DPCP_Stack[DPCP_StackCount].mutex=mutex;
									DPCP_StackCount++;
								}
								return EBUSY;
							}
							break;
#endif // YP_SUPPORT_SYNCHRONIZATION_DPCP
		// Kernelized monitor
		case YASASP_KP:		if ( !KP_Count )
							{
								KP_ScheduleOld=YASA_CURRENT_SCHEDULER->schedule;
								YASA_CURRENT_SCHEDULER->schedule=KP_Schedule;
								YASA_RESCHEDULE();
							}
							KP_Count++;
							break;
		// source code partly from  "Ismael Ripoll & Patricia Balbastre"
 		// Stack Resource Policy implementation (2001-07-20)
#ifdef YP_SUPPORT_SYNCHRONIZATION_SRP
		case YASASP_SRP:	if ( SRP_StackCount == YASA_SRP_MAXSTACK )
							{
								PRINTK("SRP: Stack full\n");
           						return EINVAL;
							}
							// compare now ceiling levels
							if ( yasa_cmpdynamicpriority2( &mutex->preemptceiling, &(SRP_Stack[SRP_StackCount].preemptceiling) ) )
							{
								SRP_StackCount++;
								SRP_Stack[SRP_StackCount].preemptceiling = mutex->preemptceiling;
								SRP_Stack[SRP_StackCount].thread		 = YASA_CURRENT_THREAD;
								yasa_logceiling(&SRP_Stack[SRP_StackCount-1].preemptceiling, &SRP_Stack[SRP_StackCount].preemptceiling);
 							}
							break;
#endif // YP_SUPPORT_SYNCHRONIZATION_SRP
		// no special or unknown synchonization protocol
		//default:			break;
	}

	if ( YASA_TEST_AND_SET_BIT( 0, &YASA_MUTEX_LOCKED(mutex) ) )
		return EBUSY;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
extern int ys_wait_wakeup (YASA_WAIT_QUEUE *waitqueue);
void yasa_wait_wakeup(YASA_WAIT_QUEUE *waitqueue)
{
	// RTLinux is using rtl_wait_wakeup
	ys_wait_wakeup( waitqueue );
}
*/

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// call is denied if mutex is unlocked !
void yasa_mutex_unlock(YASA_MUTEX *mutex)
{
	switch (mutex->protocol)
	{
		// BPIP=Basic Priority Inheritance Protocol (used in RTLinux PTHREAD_PRIO_INHERIT)
		// PIP=Priority Inheritance Protocol
		case YASASP_PIP:	YASAL_LOG_MUTEX_PRIORITY_CHANGED( YASA_CURRENT_THREAD, 0, &YASA_PRIORITY( YASA_CURRENT_THREAD ), &YASA_MUTEX_OLD_PRIORITY(mutex) );
							YASA_PRIORITY( YASA_CURRENT_THREAD ) = YASA_MUTEX_OLD_PRIORITY(mutex);
							break;

		// Priority Ceiling Protocol (used in RTLinux _RTL_POSIX_THREAD_PRIO_PROTECT)
		case YASASP_PCP:	YASAL_LOG_MUTEX_PRIORITY_CHANGED( YASA_CURRENT_THREAD, 0, &YASA_PRIORITY( YASA_CURRENT_THREAD ), &YASA_MUTEX_OLD_PRIORITY(mutex) );
							YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0, &PCP_SystemCeilingStack[PCP_SystemCeilingCount].ceiling, &PCP_SystemCeilingStack[PCP_SystemCeilingCount-1].ceiling );
							YASA_PRIORITY( YASA_CURRENT_THREAD ) = YASA_MUTEX_OLD_PRIORITY(mutex);
							PCP_SystemCeilingCount--;

							// wake up waiting threads
							while ( PCP_SystemCeilingStack[PCP_StackCount].stackpos < PCP_StackCount )
							{
								// wake up threads but do not reschedule at this moment
								PCP_StackCount--;
								YASA_MUTEX_WAKEUP( PCP_Stack[PCP_StackCount].mutex );
							};

							break;
		// CSP=Ceiling Semaphore Protocol (used in RTLinux _RTL_POSIX_THREAD_PRIO_PROTECT)
		// IPCP=Instant Priority Ceiling Protocol
		case YASASP_CSP:	YASAL_LOG_MUTEX_PRIORITY_CHANGED( YASA_CURRENT_THREAD, 0, &YASA_PRIORITY( YASA_CURRENT_THREAD ), &YASA_MUTEX_OLD_PRIORITY(mutex) );
							YASA_PRIORITY( YASA_CURRENT_THREAD ) = YASA_MUTEX_OLD_PRIORITY(mutex);
							break;


		// Dynamic Priority Ceiling Protocol
#ifdef YP_SUPPORT_SYNCHRONIZATION_DPCP
		case YASASP_DPCP:	yasa_logceiling( &DPCP_SystemCeilingStack[DPCP_SystemCeilingCount].dynamicceiling,
											 &DPCP_SystemCeilingStack[DPCP_SystemCeilingCount-1].dynamicceiling );
							DPCP_SystemCeilingCount--;
					//		yasa_dpcp_copydynamicpriority( &DPCP_MIN_PRIORITY, &mutex->dynamicceiling );
							yasa_initdynamicpriority( &mutex->dynamicceiling );

							// wake up waiting threads
							while ( DPCP_SystemCeilingStack[DPCP_StackCount].stackpos < DPCP_StackCount )
							{
								// wake up threads but do not reschedule at this moment
								DPCP_StackCount--;
								YASA_MUTEX_WAKEUP( DPCP_Stack[DPCP_StackCount].mutex );
							};
							break;
#endif //YP_SUPPORT_SYNCHRONIZATION_SRP

		// Kernelized monitor
		case YASASP_KP:		KP_Count--;
							if ( !KP_Count )
								YASA_CURRENT_SCHEDULER->schedule=KP_ScheduleOld;
							break;

		// source code partly from  "Ismael Ripoll & Patricia Balbastre"
 		// Stack Resource Policy implementation (2001-07-20)
#ifdef YP_SUPPORT_SYNCHRONIZATION_SRP
		case YASASP_SRP:	if ( SRP_StackCount && yasa_cmpdynamicpriority2( &mutex->preemptceiling, &(SRP_Stack[SRP_StackCount].preemptceiling) ))
							{
								yasa_logceiling(&SRP_Stack[SRP_StackCount].preemptceiling, &SRP_Stack[SRP_StackCount-1].preemptceiling );
								SRP_StackCount--;
							}
							break;
#endif //YP_SUPPORT_SYNCHRONIZATION_SRP
		// no special or unknown synchonization protocol
		//default:			break;
	}
	YASA_CLEAR_BIT( 0, &YASA_MUTEX_LOCKED(mutex) );
}


