//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA application startup code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/application.cpp,v 1.5 2003/01/30 09:04:46 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "application.h"
#include "workspacemanager.h"
#include "actionhandler.h"
#include "mainwindow.h"
#include "output.h"
#include "translator.h"
#include "plugins.h"
#include "helpwindow.h"
#include "settings.h"
#include "platforms/platforms.h"
#define APPLICATIONBUFFERSIZE	512


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YApplication::~YApplication()
{
	workspacemanager->CloseWorkSpace();
	closeAllWindows();

	if ( pluginmanager )
		delete pluginmanager;
	pluginmanager=0;

	if ( actionhandler )
		delete actionhandler;
	actionhandler=0;

//-> change order
	if ( workspacemanager )
		delete workspacemanager;
	workspacemanager=0;

	if (mainwindow)
		delete mainwindow;
	mainwindow=0;
//<- change order


	if (output)
		delete output;
	output=0;

	if (settings)
		delete settings;
	settings=0;

	if (translator)
		delete translator;
	translator=0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YApplication::InitInstance()
{
	int				retvalue=0;

	// Initialize Translator
	translator=new YTranslator;
	if (!translator)
		return YASAE_OUT_OF_MEMORY;
	retvalue=translator->InitInstance(this);
	if ( retvalue )
		return retvalue;

	// Initialize Settings
	settings=new YSettings();
	if (!settings)
		return YASAE_OUT_OF_MEMORY;
	retvalue=settings->InitInstance();
	if ( retvalue )
		return retvalue;

	// Initialize Output
	output=new YOutput;
	if (!output)
		return YASAE_OUT_OF_MEMORY;
	retvalue=output->InitInstance();
	if ( retvalue )
		return retvalue;

	// Initialize Workspace Manager
	workspacemanager=new YWorkSpaceManager(*this);
	if (!workspacemanager)
		return YASAE_OUT_OF_MEMORY;
	retvalue=workspacemanager->InitInstance(actionhandler);
	if ( retvalue )
		return retvalue;

	// Initialize main window and all child windows
	mainwindow=new YMainWindow;
	if (!mainwindow)
		return YASAE_OUT_OF_MEMORY;
	retvalue=mainwindow->InitInstance(this);
	if ( retvalue )
		return retvalue;
	setMainWidget(mainwindow);

	// Initialize Action Handler
	actionhandler=new YActionHandler(*workspacemanager);
	if (!actionhandler)
		return YASAE_OUT_OF_MEMORY;
	retvalue=actionhandler->InitInstance(mainwindow);
	if ( retvalue )
		return retvalue;

	// Show Mainwindow
	retvalue=mainwindow->Show();
	if ( retvalue )
		return retvalue;

	// Initialize PlugIn Manager
	pluginmanager=new YPlugInManager;
	if (!workspacemanager)
		return YASAE_OUT_OF_MEMORY;
	retvalue=pluginmanager->InitInstance();
	if ( retvalue )
		return retvalue;

	return 0;
}


#include <qmessagebox.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int main( int argc, char** argv )
{
	int				retvalue;

	QDir::setCurrent( QFileInfo( GetStartPath(argv[0]) ).dirPath() );
	YApplication	app( argc, argv );				// /

	// Correct Start path
	// Normally argv[0] contains the ptr to the full path name. But under Windows this seems to different!
	// So I implemented a plattform depended solution.
	//GetStartPath(applicationpath, APPLICATIONBUFFERSIZE, argv[0] );
	//QDir::setCurrent( QFileInfo(applicationpath).dirPath() );

	/*
	QString			a;
	a=QDir::currentDirPath();
	QMessageBox::information(0,"Meldung", a, "ok" );
	a=argv[0];
	QMessageBox::information(0,"Meldung", a, "ok" );
	*/

	retvalue=app.InitInstance();
	if ( !retvalue )
	{
		// load last/default project
		YHelpFrame::OpenHelpWindow();
		if (argc>1)
			app.GetWorkSpaceManager()->ReadWorkSpace(argv[1]);
		else
		{
			app.GetWorkSpaceManager()->NewWorkSpace();
		}
		return app.exec();									// exit to operating system, no real return
	}

	return retvalue;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YWorkSpaceManager* YApplication::GetWorkSpaceManager()
{
	if ( ! ((YApplication*)qApp)->workspacemanager )
		qWarning("YApplication::GetWorkSpaceManager() called but workspacemanager is zero!");
	return ((YApplication*)qApp)->workspacemanager;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YPlugInManager*	YApplication::GetPlugInManager()
{
	if ( ! ((YApplication*)qApp)->pluginmanager )
		qWarning("YApplication::GetPlugInManager() called but pluginmanager is zero!");
	return ((YApplication*)qApp)->pluginmanager;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YActionHandler*	YApplication::GetActionHandler()
{
	if ( ! ((YApplication*)qApp)->actionhandler )
		qWarning("YApplication::GetActionHandler() called but actionhandler is zero!");
	return ((YApplication*)qApp)->actionhandler;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YOutput* YApplication::GetOutput()
{
	if ( ! ((YApplication*)qApp)->output )
		qWarning("YApplication::GetOutput() called but output is zero!");
	return ((YApplication*)qApp)->output;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YMainWindow* YApplication::GetMainWindow()
{
	if ( ! ((YApplication*)qApp)->mainwindow )
		qWarning("YApplication::GetMainWindow() called but mainwindow is zero!");
	return ((YApplication*)qApp)->mainwindow;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTranslator* YApplication::GetTranslator()
{
	if ( ! ((YApplication*)qApp)->translator )
		qWarning("YApplication::GetTranslator() but translator is zero!");
	return ((YApplication*)qApp)->translator;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YSettings* YApplication::GetSettings()
{
	if ( ! ((YApplication*)qApp)->settings )
		qWarning("YApplication::GetSettings() but settings is zero!");
	return ((YApplication*)qApp)->settings;
}

