//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project manager
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/environment.h,v 1.4 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_ENVIRONMENT_INCLUDE
#define	YASAGUI_ENVIRONMENT_INCLUDE

#include <yasagui/yasagui.h>
#include <qobject.h>
#include "scheduler.h"
#include "actionhandler.h"
#include "logfile.h"
#include "sources.h"
#include "results.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YScheduler;
class YExecutivePlugIn;
class YOutput;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YEnvironment : public YObject
{	Q_OBJECT
	//! Ptr to output plugins
	YOutput					*output;
	//! list of schedulers used in this environment
	YSchedulerSet			schedulerset;
	//! ptr to selected executive
	YExecutivePlugIn*		executive;
	//! ptr to logfile set of this environment
	YEnvironmentLogFileSet	logfile;
	//! This is the result class
	YResults				results;
	//! default settings for property calcpreemption time (bool)
	YASA_CALCPREEMPTIONTIME	calcpreemptiontime;
	//! This environment supports tick timers
	YASA_TIMERTICK			support_timertick;
	//! tick value in this environment
	YASA_TICK				timertick;
	//! maximal number of cpu in this environment depending on executive and project settings
	YASA_MAXCPU				maxcpu;
	//! The time the current executive needs to schedule (incl. task switching)
	YASA_SCHEDULINGTIME		schedulingtime;
	//! Synchronization offset
	YASA_SYNCHRONIZATIONOFFSET
							synchronizationoffset;
	//! additional gcc (gnu c compiler) options
	YString					gcccompileroptions;
	YFilePath				logfilepath;					// YFilePath is needed to view log files !!!
	YString					logfilename;
public:						YEnvironment(const YObjectArgs &i);
	YResults &				GetResults()					{	return results;						}
	YExecutivePlugIn*		GetExecutive() const			{	return executive;					}
	YExecutivePlugIn*		SetExecutive(const QString &name);
	void					SetExecutive(YExecutivePlugIn *e);

	const YASA_CPU &		GetMaxCPU()						{	return maxcpu;						}
	YSchedulerSet&			GetSchedulerSet()				{	return schedulerset;				}
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
	int						InsertSchedulerProperty(QString &key);

	const YASA_TIMERTICK &	GetSupportTimerTick() const		{	return support_timertick;			}
	const YASA_CALCPREEMPTIONTIME &
							GetCalcPreemptionTime() const	{	return calcpreemptiontime;			}
	const YASA_TICK &		GetTimerTick() const			{	return timertick;					}
	const YASA_SCHEDULINGTIME &
							GetSchedulingTime() const		{	return schedulingtime;				}
	void					SetSchedulingTime(const YASA_TIME &t)
															{	schedulingtime=t;					}
	const YASA_SYNCHRONIZATIONOFFSET &
							GetSynchronizationOffset() const{	return synchronizationoffset;		}
	void					SetSynchronizationOffset(const YASA_TIME &t)
															{	synchronizationoffset=t;			}
	YProject*				GetProject() const				{	if ( GetSet() && GetSet()->GetParent() )
																	return ((YProject*) GetSet()->GetParent());
																return 0;							}
	const YString &			GetGCCCompilerOptions() const	{	return gcccompileroptions;			}
	void					SetGCCCompilerOptions(const QString &s)
															{	gcccompileroptions=s;				}
	QString					GetEnvironmentDirectory(bool abspath=false) const;
	YEnvironmentLogFileSet&	GetEnvironmentLogFileSet()		{	return logfile;						}
	const QString &			GetLogFileName() const			{	return logfilename.GetContent();	}
	YFilePath &				GetLogFilePath()				{	CorrectLogfilePath();
																return logfilepath;					}
	int						ExecuteEnvironment();
	void					CleanupEnvironment(QString absexecutivedir=QString::null);
	void					CorrectLogfilePath();
	//! Evaluate the logfile of the current environment
	int						EvaluateLogFile()				{	CorrectLogfilePath();
																return logfile.ParseLogFile( logfilepath.GetAbsPath() ); }
	//! This function clears the internal list of the parsed logfile
	void					CleanupLogFile()				{	logfile.CleanupLogFile();			}
signals:
	//! This signal is emitted after executing the environment
	void					EnvironmentExecuted();
	void					EnvironmentCleanedUp();
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YEnvironmentSet : public YSet
{
public:						YEnvironmentSet();
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)		{ return new YEnvironment(args);		}
	virtual YActions*		GetItemActions() const
							{ return YActionHandler::GetActions(YActionHandler::ENVIRONMENT_ACTIONS);		}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_ENVIRONMENT_INCLUDE
