//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA application action declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/filedialog.h,v 1.5 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about the actions occured by user interactions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_FILEDIALOG_INCLUDE
#define	YASAGUI_FILEDIALOG_INCLUDE

#include <qfiledialog.h>
#include "yasagui/yasagui.h"
#include "translations.h"
#include "config.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YListEntry;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFileDialog : public QFileDialog
{	Q_OBJECT
public:				YFileDialog(const QString &dir, const QString &title, const QString &filter, QFileDialog::Mode mode=QFileDialog::ExistingFile);
	int				Request(const QFileInfo &fileinfo=QFileInfo(), QFileDialog::Mode mode=QFileDialog::ExistingFile );
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFileOperations
{
public:
	static int		AddSubDirectory(QString &path, const QString &subdir);
	static QString	AddDirectory(const QString &path, const QString &subdir);
	static int		MakeRelative(QString &filename, const QString &dir=QDir::currentDirPath(), bool mode=true );
	static int		CreateDirectory(const QString &directory);
	static int		CopyFile(const QString &src, const QString &dest, bool overwrite=false);
	static int		RenameFile(const QString &src, const QString &dest, bool overwrite=false);
	static int		CopyDirectory(const QString &src, const QString &dest, bool subdir=true);
	static QString	CorrectFileName(const QString &name);
	static QString	GetFileName(const QString &path);
	static QString	CutExtension(const QString &path);
	static int		AddToFileName(QString &path, const QString &addi);
	static int		AddExtension(QString &path, const QString &addi);
	static void		RemoveDirectory(const QString &name, bool subdir=true, bool force=true);
	static int		RunCommand(const QString &name, bool showcommand=true);
	static int		RunBashCommand(const QString &name, bool showcommand=true);
	static int		RunBashScript(const QString &name, bool showcommand=true);
	static QString	ConvertToUnixPath(const QString &srcpath);

};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFile : public QFile
{
public:				YFile(const QString &f) :
					QFile(f)
					{}
	int				GetModifiedTime(QDateTime &dateedit) const;
	int				SetModifiedTime(const QDateTime &dt);
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class is used to write source/header files again and restore the modification date. The file will be loaded
//! and all important lines will be stored in the member "QStringList oldcontents". During writing the new contents
//! each important line will be compared with the contents of the list. If all contents match to <oldcontents>
//! the original modofocation date will be restored.
//! Attention: It is very important that every important line exists only once!
class YSourceFile : public YFile
{
	//! This flag indicated the how to set the modification date after writing the file
	//! If it is true the date stamp is set to current time if not it is reset to
	//! it original state (see member lastmodified).
	bool			contentchanged;
	QDateTime		lastmodified;
	QStringList		oldcontents;
public:				YSourceFile(const QString &f) :
					YFile(f),
					contentchanged(true)
					{}
					~YSourceFile()
					{
						FinalizeFile();
					}

	//! Close an opened file and reset modification date to lastmodification if contentschanged=false.
	void			FinalizeFile()
					{
						if ( isOpen() )
						{
							close();
							if (!contentchanged)
								SetModifiedTime(lastmodified);
						}
						contentchanged=true;
						oldcontents.clear();
					}
	//! Check if this is an important line
	bool			CheckImportant(const QString &string) const
					{
						return string.length()>1 && (string.left(2) != "//" );
					}
	//! Opens the file, request last modification time loads old file to stringlist, truncates the file to 0
	int				OpenFile();
	int				WriteLine(const QString &line);
	int				WriteImportant(const QString &line)
					{
						return WriteLine(line);
					}
	//! Write the specified string to YFile and makes a newline
	int				WriteUnimportant(const QString &line=QString::null )
					{
						return WriteLine(line);
					}
/*	int				WriteImportant(const QString &line);
	//! Write the specified string to YFile and makes a newline
	int				WriteUnimportant(const QString &line=QString::null )
					{
						if ( writeBlock( line.latin1(), line.length() ) == -1 )
							return YASAE_WRITE_FILE;
						if ( writeBlock( SG_NEW_LINE, 1) == -1 )
							return YASAE_WRITE_FILE;
						return 0;
					}
*/
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_FILEDIALOG_INCLUDE
