//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA list
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/list.cpp,v 1.3 2003/01/23 09:18:35 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "list.h"
#include "actionhandler.h"
#include "actions.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//YSet::YSet(const CONFIG_STRING key, const QString &caption, const QString &defaultname, YObjectArgs *args, YObject *p ) :
YSet::YSet(const CONFIG_STRING key, const QString &defaultname, const YObjectArgs &args, YObject *p ) :
YObject(args),
sectionkey(key),
parent(p),
uniquename(defaultname)
{
	setAutoDelete(TRUE);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Items will not be deleted here. Maybe they will be deleted during qlist::remove() due to switching on
//! autodelete
//! Signals "ItemRemoved" will be called before destroying the item and "ObjectChanged" will be called after
//! destroying the object
void YSet::RemoveItem(YObject *object)
{
	emit ItemRemoved(object);
	if ( GetActions() )
		GetActions()->ObjectRemoved(object);

	QList<YObject>::remove(object);
	ObjectUpdated();
//	delete item;				// will be automatically deleted if switched on ( YSet::YSet() setAutoDelete(TRUE);)
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSet::ClearSet()
{
//	while ( GetCount() )
	while ( count() )
		RemoveItem( At(0) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSet::SetParent(YObject *e)
{
	if ( parent!=e )
	{
		parent=e;
		ObjectUpdated();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSet::GetUniqueEntryName(QString &name) const
{
	if ( ! uniquename.isEmpty() )
		name=uniquename.arg( GetCount()+1 );
	else
		name=QString::number( GetCount()+1 );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! SetConfig is used here to add or configure the elements of the set. Signals "ObjectChanged" and "NewItem" will be sent
//int YSet::CreateNewItem(const QString &n)
YObject* YSet::CreateNewItem(const QString &n)
{
	YObject		*object;
	QString		name;

	name=n;
	if ( name.isEmpty() )
	{
		GetUniqueEntryName(name);
	}

	object = AllocNewItem( YObjectArgs(name, GetItemActions(), this ) );
	if ( object )
	{
		AppendItem(object);		//QList<YObject>::append(object);

		// inform parent of YList that this list was changed
		connect( object, SIGNAL(UpdateObject(YObject *)), this, SLOT( ItemChanged(YObject *) ) );

		// emit signal "New Item" to every associated object
		emit NewItem(object);
		ObjectUpdated();

		// emit signal to YActions that an object of this type was created
		if ( GetActions() )
			GetActions()->NewObjectCreated(object);
		return object;
	}
	else
		return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Fills the given string list with the names of the list item. There are
//! strings returned by GetName() used.
int YSet::FillStringList(QStringList &list, bool sort)
{
	for (unsigned int i=0; i<GetCount();i++)
	{
		list.append( At(i)->GetName() );
	}
	if (sort)
		list.sort();
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//\return Returns the index number of the given object. If no object was found
//\return -1 is returned.
/*
unsigned int YSet::FindItemIndex(const YObject *object)
{
	//findRef(object); // crashed
	for (unsigned int i=0; i<GetCount();i++)
	{
		if ( object == At(i) )
			return i;
	}
	return -1;
}
*/

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a slot which is called after changing an item in this list. It sends the signal "UpdateItem(YObject *item)" first and
//! then UpdateObject(YSet *this) to signal a change of the whole list
//! It is not called after creating or destroying an item!
void YSet::ItemChanged(YObject *object)
{
	emit UpdateItem(object);
	ObjectUpdated();
}


