//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA list
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/list.h,v 1.5 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_LIST_INCLUDE
#define	YASAGUI_LIST_INCLUDE

#include <yasagui/yasagui.h>
#include <qobject.h>
#include <qlist.h>
#include <qstrlist.h>
#include "datatypes.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YInputParser;
class YOutputParser;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** This is the base class for lists in the application.
//  Attention: use the function remove() to remove and delete a list entry!!! If "autodelete" is not switched on,
//  the item will be deleted automatically if they are removed from the list!
**/
class YSet :  public YObject, protected QList<YObject>
{	Q_OBJECT
	//! This is the configuration identifier e.g. "PROJECT", "EXECUTIVE"
	const CONFIG_STRING		sectionkey;
	//! Ptr to an parent object, which contains several lists
	YObject					*parent;
	//! This is a part of the default name e.g. "project_%1"
	QString					uniquename;
//	bool					childnotify;						//! If true the YSet is able to receive signals from childs, if they are changed and they will be forwarded to SIGNAL(ItemChanged)
public:						YSet(const CONFIG_STRING key, const QString &defaultname, const YObjectArgs &args=YObjectArgs(), YObject *p=0 );
	void					RemoveItem(YObject *item);
	void					GetUniqueEntryName(QString &) const;
	//! Return the section key for configuration issues or NULL if not set (default)
	const CONFIG_STRING		GetSectionKey() const					{ return sectionkey;					}
	YObject*				CreateNewItem(const QString &name=QString::null );
	virtual int				SetConfig(YInputParser &parser)			{ return YObject::SetConfig(parser);	}
	virtual int				GetConfig(YOutputParser &parser)		{ return YObject::GetConfig(parser);	}
	void					ClearSet();
	void					SetParent(YObject *);
	//! Returns the parent object of this list, if used
	YObject*				GetParent()	const						{ return parent;						}

	//virtual unsigned int	GetCount() const						{ return QList<YObject>::count();		}
	//! Returns the number of items in the list
	unsigned int			GetCount() const						{ return QList<YObject>::count();		}
	//! Returns the item at index n or NULL if the item could not be found
	virtual YObject*		At(unsigned int n) 						{ return QList<YObject>::at(n);			}
//	void					EnableChildNotify(bool state=true)		{ childnotify=state;					}
	int						FillStringList(QStringList &list, bool sort=true);
	//! Returns the index number of the given object. If no object was found
	//\return Index of item or -1 if no item could be found
	unsigned int			FindItemIndex(const YObject *object)	{ return findRef(object);				}

protected:
	//! This item is fully initialized and should be appended to the list
	virtual void			AppendItem(YObject *object)				{ if (object)
																		QList<YObject>::append(object);		}
	//! This function is provided for convenience to get the YAction Ptr of derived lists during
	//! CreateNewItem(). Default=0
	virtual YActions*		GetItemActions() const					{ return 0;								}
	//! Item will be allocated but not inserted into list. This function is provied for convenience
	//! to enable derived lists to create its own derived list objects.
	virtual YObject*		AllocNewItem(const YObjectArgs &id) =0;
signals:
	//! Signals that a child item was changed.
	void					UpdateItem(YObject *);
	//! Signals that a new child item was created and was inserted into the list
	void					NewItem(YObject *object);
	//! Signals that a child item should be destroyed. Currently the item is already in list
	//! and will be removed and destroyed if the signal handling is completed
	void					ItemRemoved(YObject *object);
public slots:
	//! This slot is called every time a child emits the signal YObject::UpdateObject()
	void					ItemChanged(YObject *);

};





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_LIST_INCLUDE
