//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA mutex window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/mutexwindow.h,v 1.4 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about mutex windows
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_MUTEXWINDOW_INCLUDE
#define	YASAGUI_MUTEXWINDOW_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "mainwindow.h"
#include "widgets.h"
#include "task.h"
#include "mutex.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the mutex configuration
class YMutexFrame : public YObjectFrame
{	Q_OBJECT
	QScrollView			scrollview;
public:					YMutexFrame(QWidget *parent, YFrameArgs &args) :
						YObjectFrame(args),
						scrollview(parent)
						{}
	virtual int			InitInstance();
	virtual QWidget*	GetMainWidget()						{ return &scrollview;	}
	static YFrame*		CreateMutexFrame(QWidget *parent, YFrameArgs &args)
						{
							return new YMutexFrame( parent, args );
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProtocolWidget : public YComboWidget
{	Q_OBJECT
public:					YProtocolWidget(YMutex *mutex, QWidget *parent) : YComboWidget(mutex, parent)
						{	insertItem(MSGP_MUTEX_PTYPE_NORMAL);
							insertItem(MSGP_MUTEX_PTYPE_PIP);
							insertItem(MSGP_MUTEX_PTYPE_CSP);
							insertItem(MSGP_MUTEX_PTYPE_PCP);
							insertItem(MSGP_MUTEX_PTYPE_DPCP);
							insertItem(MSGP_MUTEX_PTYPE_KP);
							insertItem(MSGP_MUTEX_PTYPE_SRP);
#if YASASP_MAX!=6
#error inconsistency occured !
#error Insert/Remove new item to combobox here !!!
#endif
							UpdateWidget();
						}
public slots:
	virtual void		UpdateWidget()							{ if ( currentItem()!=((YMutex*)object)->GetProtocol() )
																	setCurrentItem( ((YMutex*)object)->GetProtocol());	}
	virtual void		UpdateValue(int value)					{ ((YMutex*)object)->SetProtocol( value );		}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YMutexTimeListViewItem : public YListViewItem
{	Q_OBJECT
public:					YMutexTimeListViewItem(YObject *o, QListView *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{ }
						YMutexTimeListViewItem(YObject *o, QListViewItem *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{ }
	virtual int			InitInstance();
	void				UpdateObject(YObject *);
protected:
	QString				key(int column, bool ascending) const
						{
							YTask	*task;
							if ( object && object->GetSet() && object->GetSet()->GetParent() )
							{
								task= (YTask*) object;
								if ( task )
								{
									QString	tmp;
									switch ( column)
									{
									case 1:	return YString::Justify( task->GetPeriod().GetVisibleContentString() );
									case 2:	return YString::Justify( task->GetPeriod().GetVisibleContentString() );
									case 3:	return YString::Justify( task->GetDeadline().GetVisibleContentString() );
									case 4:	return YString::Justify( task->GetLaxity().GetVisibleContentString() );
									}
								}
							}
							return text(column);
							Q_UNUSED(ascending);
						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YMutexTimesListView : public YListView
{	Q_OBJECT
	YMutex				*mutex;
	YMutexTimeListViewItem* _FindItem(YTask *task);
public:					YMutexTimesListView(YTaskSet *s, QWidget *parent, YMutex *m) :
						YListView(s, parent, MSGP_MUTEX_TASKS),
						mutex(m)
						{}
	virtual	YMutexTimeListViewItem* CreateNewItem(QListViewItem *parent, YTask *object, QListViewItem *after=0)
						{
							if ( parent )
								return new YMutexTimeListViewItem(object,parent,after);
							else
								return new YMutexTimeListViewItem(object,this,after);
						}
	virtual int			CreateSetItems();							//! Create listview item again/new
	virtual int			InitInstance();
public slots:
	void				UpdateTaskProperties(YObject *task);
	virtual void		NewChildObject(YObject *object);			//! A new YTask was created
	void				ResourceActionRemoved(YObject *);			//! A resource action was removed
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_MUTEXWINDOW_INCLUDE
