//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/project.cpp,v 1.3 2003/01/23 09:18:37 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "project.h"
#include "actionhandler.h"
#include "filedialog.h"
#include "workspace.h"
#include "parser.h"
#include "application.h"
#include "output.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YProjectSet::YProjectSet() :
YSet(YPC_PROJECT, MSGP_DEFAULT_PROJECTNAME, YObjectArgs(MSGP_CAPTION_PROJECTS, YActionHandler::GetActions(YActionHandler::PROJECTSET_ACTIONS)))
{}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! All project work directories will be cleaned up
void YProjectSet::CleanupProjects()
{
	for (unsigned int i=0; i< GetCount(); i++)
	{
		((YProject*) At(i) )->CleanupProject();
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YProject::YProject(const YObjectArgs &args)
: YObject(args),
  taskset(0),
  results( YResultArgs(YResultArgs::PROJECT_RESULTS) ),
  projectdir( YFileOperations::CorrectFileName(args.GetName() ) ),
  calcpreemptiontime((bool)SUPPORT_CALC_PREEMPTIONTIME),
  endtime(30000)
//  tickvalue(PROJECT_TICK_VALUE)
{
	results.SetParent(this);
	projectlogfile.SetParent(this);
	taskset.SetParent(this);
	mutexset.SetParent(this);
	environmentset.SetParent(this);
	sourceset.SetParent(this);
	headerset.SetParent(this);

	connect( &maxcpu			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &calcpreemptiontime, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &timertick			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &projectdir		, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProject::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_PROJECT_DIR) )
		return parser.ReadContentObject(projectdir);
	if ( parser.CheckSectionStart(YPC_NUMBER_OF_CPUS) )
		return parser.ReadContentObject(maxcpu);
	if ( parser.CheckSectionStart(YPC_SOURCE ) )
		return parser.NewSectionSet(YPC_SOURCE, sourceset );
	if ( parser.CheckSectionStart(YPC_HEADER ) )
		return parser.NewSectionSet(YPC_HEADER, headerset );
	if ( parser.CheckSectionStart(YPC_CALC_PREEMPTIONTIME) )
		return parser.ReadContentObject(calcpreemptiontime );
	if ( parser.CheckSectionStart(YPC_TIMER_TICK) )
		return parser.ReadContentObject(timertick);
	if ( parser.CheckSectionStart(YPC_END_TIME) )
		return parser.ReadContentObject(endtime);
	if ( parser.CheckSectionStart(YPC_TASK ) )
		return parser.NewSectionSet(YPC_TASK, taskset );
	if ( parser.CheckSectionStart(YPC_MUTEX ) )
		return parser.NewSectionSet(YPC_MUTEX, mutexset );
	if ( parser.CheckSectionStart(YPC_ENVIRONMENT ) )
		return parser.NewSectionSet(YPC_ENVIRONMENT, environmentset);
	if ( parser.CheckSectionStart(YPC_RESULTS) )
		return parser.NewSection(YPC_RESULTS, results );

	return YObject::SetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YProject::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	retvalue=parser.WriteContentObject(YPC_PROJECT_DIR, projectdir);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_NUMBER_OF_CPUS, maxcpu);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_CALC_PREEMPTIONTIME, calcpreemptiontime);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_TIMER_TICK, timertick);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_END_TIME, endtime);
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_TASK, taskset );
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_HEADER, headerset );
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_SOURCE, sourceset );
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_MUTEX, mutexset );
		if ( retvalue )	return retvalue;
	retvalue=parser.NewSectionSet(YPC_ENVIRONMENT, environmentset );
		if ( retvalue )	return retvalue;
	if ( environmentset.GetCount() )
	{
		retvalue=parser.NewSection(YPC_RESULTS, results );
			if ( retvalue )	return retvalue;
	}

	return YObject::GetConfig(parser);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YProject::SetProjectDir(const QString &pd)
{
	if ( projectdir != YFileOperations::CorrectFileName(pd) )
	{
		RemoveProjectDirectory();
		projectdir=YFileOperations::CorrectFileName(pd);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This functions returns the project directory. The directory will be created if it does not exist
//! Attention: The current directory must be set to the $YASA Directory
//\input YListEntry = ptr to a project
//\return 'dir' is set to the relative path to the project from the view of $WORKSPACE (if possible) otherwise dir is untouched
//! example: Yasa_Workspace: /home/user/yasa/workspaces/project_1
//! example: This function will return "./project_1" or "project"
QString YProject::CopyProjectDir(bool absdir)
{
	if ( this && GetSet() )
	{
		QString dir;
		dir=QString::null;
		if ( absdir )
		{
			YWorkSpace*	workspace;
			workspace = (YWorkSpace*) GetSet()->GetParent();
			//dir=workspace->GetFileInfo().dirPath(true);
			dir=workspace->GetAbsFilePath();
		}
		YFileOperations::AddSubDirectory(dir, projectdir.GetContentString() );
		YFileOperations::CreateDirectory(dir);
		return dir;
	}
	qWarning("YProject::GetProjectDir() with illegal arguments called. That should not be happened!" );
	return QString::null;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The selected project will be deleted from the workspace. This function ensures that
//! the whole directory will be removed and all environments are cleanuped
void YProject::RemoveProjectDirectory()
{
	YApplication::GetOutput()->Message( );
	YApplication::GetOutput()->Message( MSGP_REMOVING_PROJECT.arg( GetName() ) );
	CleanupProject();
	YApplication::GetOutput()->Message( MSGP_E_REMOVING_DIRECTORY.arg( CopyProjectDir(true) ) );
	YFileOperations::RemoveDirectory( CopyProjectDir(true) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! All environments of the selected project will be cleaned up
void YProject::CleanupProject()
{
	for (unsigned int i=0; i<environmentset.GetCount(); i++)
	{
		((YEnvironment*)environmentset.At(i) )->CleanupEnvironment();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Parse all logfiles. If one is correct. Do not return an error.
int YProject::EvaluateLogFiles()
{
	int	retvalue2, retvalue=YASAE_NO_MORE_ENTRIES;
	for (unsigned int i=0; i<environmentset.GetCount(); i++)
	{
		retvalue2 = ((YEnvironment*)environmentset.At(i) )->EvaluateLogFile();
		if ( retvalue )
			retvalue=retvalue2;
	}
	return retvalue;
}

