//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/projectwindow.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about project windows
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_PROJECTWINDOW_INCLUDE
#define	YASAGUI_PROJECTWINDOW_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "mainwindow.h"
#include "widgets.h"
#include "project.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This window is showing the project configuration
class YProjectFrame : public YObjectFrame
{	Q_OBJECT
	QScrollView			scrollview;
public:					YProjectFrame(QWidget *parent, YFrameArgs &args) :
						YObjectFrame(args),
						scrollview(parent)
						{}
	virtual int			InitInstance();
	virtual QWidget*	GetMainWidget()								{ return &scrollview;	}
	static YFrame*		CreateProjectFrame(QWidget *parent, YFrameArgs &args)
						{
							return new YProjectFrame(parent, args);
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YCPUProjectWidget : public YIntegerWidget
{	Q_OBJECT
public:					YCPUProjectWidget(YProject *project, QWidget *parent) : YIntegerWidget(project, parent)
						{	setMinValue( project->GetMaxCPU().GetMinValue() );
							setMaxValue(YG_MAX_CPU);
							UpdateWidget();
						}
public slots:
	virtual void		UpdateWidget()				{ //int a=((YProject*)object)->GetMaxCPU();
														if ( value()!=((YProject*)object)->GetMaxCPU() )
														setValue( ((YProject*)object)->GetMaxCPU());	}
	virtual void		UpdateValue(int value)		{ if ( value != 0)
														((YProject*)object)->SetMaxCPU( value );		}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YEndTimeProjectWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YEndTimeProjectWidget(YProject *project, QWidget *parent) : YYasaTimeWidget(project, parent)
						{	UpdateWidget();
						}
public slots:
	virtual void		UpdateWidget()				{ if ( GetYasaTime() != ((YProject*)object)->GetEndTime() )
														SetYasaTime( ((YProject*)object)->GetEndTime());	}
	virtual void		UpdateValue(YASA_TIME value) { ((YProject*)object)->SetEndTime( value );			}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTickProjectWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YTickProjectWidget(YProject *project, QWidget *parent) : YYasaTimeWidget(project, parent)
						{	UpdateWidget();
							updateDisplay();	}
public slots:
	virtual void		UpdateWidget()				{ if ( GetYasaTime() != ((YProject*)object)->GetTimerTick() )
														SetYasaTime( ((YProject*)object)->GetTimerTick());	}
	virtual void		UpdateValue(YASA_TIME value) { ((YProject*)object)->SetTimerTick( value );			}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v==YTIME::YASA_TIME_MIN)	return QString( MSGP_ENVIRONMENT_TIMERTICK_NO );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_ENVIRONMENT_TIMERTICK_NO )	return YTIME::YASA_TIME_MIN;
																	return text().toInt(ok);						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YCalcPreemptionTimeProjectWidget : public YCheckBoxWidget
{	Q_OBJECT
public:					YCalcPreemptionTimeProjectWidget(YProject *project, QWidget *parent) :
						YCheckBoxWidget(project, parent)
						{	setChecked( project->GetCalcPreemptionTime() );
							UpdateWidget();
						}
public slots:
	virtual void		UpdateWidget()				{ if ( isChecked()!=((YProject*)object)->GetCalcPreemptionTime() )
														setChecked( ((YProject*)object)->GetCalcPreemptionTime());	}
	virtual void		UpdateValue(bool value)		{ ((YProject*)object)->SetCalcPreemptionTime( value );		}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProjectPathWidget : public YStringWidget
{	Q_OBJECT
public:					YProjectPathWidget(YObject *object, QWidget *parent) :
						YStringWidget(object,parent) {	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()							{ if ( text()!=((YProject*)object)->GetProjectDir().GetContentString() )
																	setText(((YProject*)object)->GetProjectDir().GetContentString() );	}
	virtual void		UpdateValue(const QString &t)			{ ((YProject*)object)->SetProjectDir( t );		}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_PROJECTWINDOW_INCLUDE
