//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project, scheduler support
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/scheduler.cpp,v 1.4 2003/01/30 09:29:55 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "scheduler.h"
#include "actionhandler.h"
#include "parser.h"
#include "plugins.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YSchedulerSet::YSchedulerSet() :
YSet(YPC_SCHEDULER, MSGP_DEFAULT_SCHEDULERNAME, YObjectArgs( MSGP_CAPTION_SCHEDULERS, YActionHandler::GetActions(YActionHandler::SCHEDULERSET_ACTIONS)))
{
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSchedulerSet::InitInstance()
{
	connect( & (((YEnvironment*)GetParent())->GetMaxCPU()), SIGNAL( UpdateObject() ), this, SLOT( MaxCPUChanged() ) );
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSchedulerSet::MaxCPUChanged()
{
	EnsureItemCount( (((YEnvironment*)GetParent())->GetMaxCPU()) );
	//ObjectUpdated();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Create scheduler item for each CPU
int YSchedulerSet::EnsureItemCount(unsigned int maxcpu)
{
	unsigned int i;
	if ( GetCount() > maxcpu )
	{
		while( GetCount() > maxcpu )
		{
			RemoveItem( At( GetCount()-1 ) );
		}
		return 0;
	}
	YScheduler	*scheduler;
	for (i=GetCount(); i<maxcpu; i++)
	{
		scheduler=(YScheduler*)CreateNewItem();
		if ( ! scheduler )
			return YASAE_OUT_OF_MEMORY;
	}
	return 0;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YScheduler::YScheduler(const YObjectArgs &args=YObjectArgs() ) :
YObject(args),
scheduler(0),
calcpreemptiontime( & ( GetEnvironment()->GetCalcPreemptionTime() ) ),
support_timertick( & ( GetEnvironment()->GetSupportTimerTick() ) ),
timertick( & GetEnvironment()->GetTimerTick() )
{
	connect( &support_timertick			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &calcpreemptiontime		, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &timertick					, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read scheduler specific configuration data from instance and set to stream
int YScheduler::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_SCHEDULERNAME) )
	{
		QString	name;
		int retvalue = parser.ReadContentString( name );
		if ( ! retvalue )
		{
			if ( SetScheduler(name) == 0)
					retvalue=YASAE_ERROR_IN_ARGUMENTS;
				return retvalue;
		}
	}
	if ( parser.CheckSectionStart(YPC_CALC_PREEMPTIONTIME) )
		return parser.ReadContentObject(calcpreemptiontime);
	if ( parser.CheckSectionStart(YPC_TIMER_TICK) )
		return parser.ReadContentObject(timertick);
	return YObject::SetConfig(parser);
	//return parser.SectionEnd();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read scheduler specific configuration data from stream and set to instance
int YScheduler::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	if ( scheduler )
	{
		retvalue=parser.WriteContentString(YPC_SCHEDULERNAME, scheduler->GetShortName() );
			if ( retvalue )	return retvalue;
	}
	retvalue=parser.WriteContentObject(YPC_CALC_PREEMPTIONTIME, calcpreemptiontime);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_TIMER_TICK, timertick);
		if ( retvalue )	return retvalue;
	return YObject::GetConfig(parser);
	//return parser.SectionEnd();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new executive name was set
YSchedulerPlugIn* YScheduler::SetScheduler(const QString &name)
{
	YSchedulerPlugIns	&schedulerset=YSchedulerPlugIns::GetSchedulerPlugIns();
	for (unsigned i=0; i<schedulerset.GetCount(); i++)
	{
		if ( name == ((YPlugIn*)schedulerset.At(i))->GetShortName() ||
			 name == ((YPlugIn*)schedulerset.At(i))->GetFullName() )
		{
			SetScheduler( (YSchedulerPlugIn*)schedulerset.At(i) );
			return scheduler;
		}
	}
	SetScheduler( 0 );	// unset scheduler
	return 0;			// Object not found -> executive undefined
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YScheduler::SetScheduler(YSchedulerPlugIn *s)
{
	if ( s!=scheduler )
	{
		scheduler=s;
		if ( s )
		{
			// does the scheduler support calc preemption time?
			if ( scheduler->GetSupportCalcPreemptionTime() )
				calcpreemptiontime.Invalid();
			else
				calcpreemptiontime=false;
			calcpreemptiontime.SetChildsDisabled( ! scheduler->GetSupportCalcPreemptionTime() );

			//calcpreemptiontime=scheduler->GetSupportCalcPreemptionTime();

			if ( ! scheduler->GetSupportTimerTick() )
				support_timertick=false;
			else
				support_timertick.Invalid();
			support_timertick.SetChildsDisabled( ! scheduler->GetSupportTimerTick() );
			//support_timertick=scheduler->GetSupportTimerTick();

			timertick.SetChildsDisabled( ! scheduler->GetSupportTimerTick() );
			timertick.SetMinTimerTick( scheduler->GetMinTimerTick() );
		}
		else
		{
			calcpreemptiontime.Invalid();
			timertick.SetMinTimerTick( YTIME::YASA_TIME_MIN );
		}

		ObjectUpdated();
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YEnvironment* YScheduler::GetEnvironment() const
{
	if ( GetSet() && GetSet()->GetParent() )
	{
		return ((YEnvironment*)GetSet()->GetParent());
	}
	return 0;
}
