//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project, scheduler support
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/scheduler.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_SCHEDULER_INCLUDE
#define	YASAGUI_SCHEDULER_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasagui/yasagui.h>
#include <qobject.h>
#include "list.h"
#include "actionhandler.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerPlugIn;
class YProject;
class YEnvironment;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class represents an instance or the scheduler in the specific
//! environment configuration.
class YScheduler : public YObject
{	Q_OBJECT
	YSchedulerPlugIn		*scheduler;					//! ptr to scheduler plugin
	YASA_CALCPREEMPTIONTIME	calcpreemptiontime;			//! should the preemption time be calculated
	YASA_TIMERTICK			support_timertick;			//! This environment supports tick timers
	YASA_TICK				timertick;					//! timer tick of the scheduler
public:						YScheduler(const YObjectArgs &args);
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
	const YASA_TIMERTICK &	GetSupportTimerTick() const			{ return support_timertick;			}
	const YASA_CALCPREEMPTIONTIME&
							GetCalcPreemptionTime() const		{ return calcpreemptiontime;		}
	//! Return true if all objects (executive & scheduler) are able to support calc preemption time
	//! If they are not able all childs are disabled
	bool					GetSupportCalcPreemptionTime() const{ return ! calcpreemptiontime.AreChildsDisabled(); }
	void					SetCalcPreemptionTime(bool b)		{ calcpreemptiontime=b;				}
	void					DeriveCalcPreemptionTime()			{ calcpreemptiontime.Invalid();		}
	const YASA_TICK&		GetTimerTick() const				{ return timertick;	}
	void					SetTimerTick(YASA_TIME t)			{ timertick=t;						}
	void					DeriveTimerTick()					{ timertick.Invalid();				}
	YSchedulerPlugIn*		GetScheduler() const				{ return scheduler;					}
	void					SetScheduler(YSchedulerPlugIn *);
	YSchedulerPlugIn*		SetScheduler(const QString &);
	YEnvironment*			GetEnvironment() const;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerSet : public YSet
{	Q_OBJECT
public:						YSchedulerSet();
	int						EnsureItemCount(unsigned int i);
	int						InitInstance();
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)	{ return new YScheduler(args);	}
	virtual YActions*		GetItemActions() const
							{ return YActionHandler::GetActions(YActionHandler::SCHEDULER_ACTIONS);	}
public slots:
	void					MaxCPUChanged();
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_SCHEDULER_INCLUDE
