//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA settings
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/settings.cpp,v 1.3 2003/01/23 09:18:38 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file handles the global settings
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qtextview.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qcursor.h>
#include "qbitmap.h"
#include "settings.h"
#include "application.h"
#include "filedialog.h"
#include "output.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Insert a figure to the internal list. The figure will not be loaded! It will be loaded if it is used
//! the first time. filedir can be a relative path from $YASA
void YPixmapSettings::InsertFigure(const QString &filedir, const QColor &color)
{
	pixmaps.append( new YPixmap( YFileOperations::AddDirectory( ((YApplication*) qApp)->GetYasaPath() , filedir ), color, GetPixmapArgs() ) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Inserts a brush to the internal figure list. This brush is valid after calling this function
//void YPixmapSettings::InsertBrush(const QColor &color, QBrush::BrushStyle style)
void YPixmapSettings::InsertBrush(const QBrush &brush, QBrush::BrushStyle style)
{
	pixmaps.append( new YPixmap( brush, style, GetPixmapArgs() ) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Set the backdrop of a textview (It does not work with non-derived scollviews! )
void YApplicationFigures::SetTextViewBackdrop(QTextView &tv, YApplicationFigures::PIXMAP_TYPE type)
{
	YSettings	*settings=((YApplication*)qApp)->GetSettings();
	YPixmap		*pixmap=settings->GetApplicationFigures().pixmaps.at(type);

	if ( pixmap && pixmap->GetBrush() )
	{
		tv.setPaper( *pixmap->GetBrush() );
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Set backdrop of listview
void YApplicationFigures::SetListViewBackdrop(QListView &lv, YApplicationFigures::PIXMAP_TYPE type)
{
	YSettings	*settings=((YApplication*)qApp)->GetSettings();
	YPixmap		*pixmap=settings->GetApplicationFigures().pixmaps.at(type);

	if ( pixmap && pixmap->GetBrush() )
	{
		lv.setBackgroundMode( QWidget::PaletteBackground );
		{
			QPalette p = lv.palette();
			p.setBrush( QPalette::Active, QColorGroup::Base, *pixmap->GetBrush() );
			lv.setPalette( p );
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Set the backdrop of a textview (It does not work with non-derived scollviews! )
void YApplicationFigures::SetIcon(QMainWindow &mv, YApplicationFigures::PIXMAP_TYPE type)
{
	YSettings	*settings=((YApplication*)qApp)->GetSettings();
	YPixmap		*pixmap=settings->GetApplicationFigures().pixmaps.at(type);

	if ( pixmap && pixmap->GetPixmap() )
	{
		mv.setIcon( *pixmap->GetPixmap() );
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Set the backdrop of a textview (It does not work with non-derived scollviews! )
void YApplicationFigures::SetIconSet(QAction &a, YApplicationFigures::PIXMAP_TYPE type)
{
	YSettings	*settings=((YApplication*)qApp)->GetSettings();
	YPixmap		*pixmap=settings->GetApplicationFigures().pixmaps.at(type);

	if ( pixmap && pixmap->GetPixmap() )
	{
		a.setIconSet( QIconSet( *pixmap->GetPixmap() ) );
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Create a cursor e.g. mouse pointers
int YApplicationFigures::CreateCursor(QCursor **c, YApplicationFigures::PIXMAP_TYPE type)
{
	YSettings	*settings=((YApplication*)qApp)->GetSettings();
	YPixmap		*pixmap=settings->GetApplicationFigures().pixmaps.at(type);

	if ( pixmap && pixmap->GetPixmap() )
	{
		QPixmap	newpixmap( *pixmap->GetPixmap() );
		newpixmap.setMask( newpixmap.createHeuristicMask() );		// get cursor mask
		*c=new QCursor ( newpixmap );
		return 0;
	}
	*c=0;
	return YASAE_COULD_NOT_OPEN_FILE;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Load a pixmap if needed
int YPixmap::LoadPixmap()
{
	int	retvalue=YASAE_ERROR_IN_ARGUMENTS;

	// pixmap already initialized ?
	if ( IsPixmapValid() )
		return 0;
	if ( filename.length() )						// filename valid ?
	{
		retvalue=YASAE_OUT_OF_MEMORY;
		pixmap=new QPixmap(filename);
		if ( pixmap )
		{
			if ( ! pixmap->isNull() )				// pixmap loaded ?
			{
				brush.setPixmap(*pixmap);
				retvalue=0;
			}
			else
			{
				delete pixmap;
				pixmap=0;
				if (warn_if_failed)
					((YApplication*)qApp)->GetOutput()->Warning( MSGP_BACKGROUND_NOT_FOUND.arg(filename) );
				retvalue=YASAE_COULD_NOT_OPEN_FILE;
			}
		}
	}
	if (retvalue)
		tried_n_failed=true;
	else
		tried_n_failed=false;
	return retvalue;
}


