//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA source generator
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/sourcegenerator.cpp,v 1.8 2003/01/30 09:08:08 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qdir.h>
#include "project.h"
#include "sourcegenerator.h"
#include <qstringlist.h>
#include "output.h"
#include "application.h"
#include "workspace.h"
#include "plugins.h"
#include "sources.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Create project sources, copy makefiles and make needed scripts (runit, cleanup)
int YSourceGenerator::PrepareWorkProject()
{
	QString				srcfile;
	QString				tmp;
	QString				currentdir;
	QString				absworkdir;
	QString				absprojectdir;
	YProject			*project=environment->GetProject();
	int					retvalue=YASAE_ERROR_IN_ARGUMENTS;

	if ( environment && environment->GetExecutive() && project)
	{
		// print feature list
		output->Comment( MSGP_E_EXECUTIVE_SUPPORT.arg( environment->GetExecutive()->GetName() ) );
		QStringList	list=environment->GetExecutive()->GetFeatureList();
		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
			output->Comment( (*it) );

		// Get current directory
		currentdir=QDir().absPath();
		tmp=MSGP_E_GENERATING_SOURCECODE.arg( project->GetName() );
		tmp=tmp.arg( environment->GetName() );
		output->Message( tmp.arg( environment->GetExecutive()->GetName() ) );

		// Create work directory
		absprojectdir=project->CopyProjectDir(true);
		absworkdir=environment->GetEnvironmentDirectory(true);

		if ( ! QDir::setCurrent(absworkdir) )
		{
			output->Error( MSGP_EE_CREATE_WORK_DIRECTORY.arg(absworkdir) );
			retvalue=YASAE_COULD_NOT_OPEN_FILE;
		}
		else
		{	// change dir to work directory and create sources
			output->Message( MSGP_E_CHANGING_DIRECTORY.arg(absworkdir) );
			retvalue=CreateProjectSources();
			if ( ! retvalue )
			{
				// Copy the following files to the current directory (current environment)
				CopyProjectFile( SOURCE_MAKEFILE_NAME );
				CopyProjectFile( SOURCE_MAKEFILE_PROJECT_NAME );
				CopyProjectFile( SOURCE_MAKEFILE_SCHEDULERS_NAME );
				CopyProjectFile( SOURCE_RUNPROJECT_NAME );
				CopyProjectFile( SOURCE_CVSIGNORE_NAME );
				YFileOperations::RenameFile(SOURCE_CVSIGNORE_NAME, SOURCE_CVSIGNORE_FINAL_NAME, true );

				// create start file "runit"
				retvalue=CreateScriptFile(absprojectdir, absworkdir, SG_MAKE_COMMAND_SIMULATE, SOURCE_RUNIT_NAME);
				if ( ! retvalue )
				{
					retvalue=CreateScriptFile(absprojectdir, absworkdir, SG_MAKE_COMMAND_CLEANUP, SOURCE_CLEANUP_NAME);
					if ( ! retvalue )
						retvalue=CreateScriptFile(absprojectdir, absworkdir, SG_MAKE_COMMAND_UNINSTALL, SOURCE_UNINSTALL_NAME);
				}

				// Check for valid environment names
				for (unsigned int i=0; i<environment->GetSchedulerSet().GetCount(); i++)
				{
					YSchedulerPlugIn	*scheduler;
					scheduler=((YScheduler*)environment->GetSchedulerSet().At(i))->GetScheduler();
					if ( scheduler && ( scheduler->GetShortName() == environment->GetName() ) )
					{
						output->Warning( MSGP_SG_CHANGE_ENVIRONMENT_NAME.arg(environment->GetName() )
																		.arg(scheduler->GetFullName() )
																		.arg(scheduler->GetShortName() ) );
					}
				}

			}
			else
				output->Error( MSGP_EE_GENERATING_ABORTED.arg( retvalue ) );

			// restore original directory
			QDir::setCurrent( currentdir );
		}
	}
	else
		output->Error(MSGP_EE_EXECUTIVE_NOT_DEFINED);
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Copies a file from $EXECUTIVE/files/ to $PROJECTWORKDIR/
int	YSourceGenerator::CopyProjectFile(const QString &filename)
{
	YProject			*project=environment->GetProject();
	QString				srcfile;
	srcfile=((YWorkSpace*) project->GetSet()->GetParent() )->GetApplication().GetStartPath();
	srcfile=YFileOperations::AddDirectory( srcfile, environment->GetExecutive()->GetPath() );
	srcfile=YFileOperations::AddDirectory( srcfile, SOURCE_FILES );
	srcfile=YFileOperations::AddDirectory( srcfile, filename );
	if ( QFile(srcfile).exists() )
		return YFileOperations::CopyFile(srcfile, filename);
	else
		return YASAE_COULD_NOT_OPEN_FILE;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Creates a script file to start or cleanup the environment
int	YSourceGenerator::CreateScriptFile(const QString &projectdir, const QString &workdir, const QString &command, const QString &filename)
{
	QString				str, tmp, tmp2;
	YSourceFile			file(filename);
	int					retvalue;
	YProject			*project=environment->GetProject();
	YFilePath			*source;
	YSchedulerPlugIn	*scheduler;

	retvalue=file.OpenFile();
	if (! retvalue )
	{
		// write script header
		file.WriteUnimportant(SG_SCRIPT_STARTUP);
		// write license
		file.WriteUnimportant(MSGP_SG_GPL_LICENSE_BASH_SCRIPT);
		// write YASA
		str=SG_SHELLVAR_YASA;
		tmp=((YWorkSpace*) project->GetSet()->GetParent() )->GetApplication().GetStartPath();
		file.WriteImportant(str.arg( YFileOperations::ConvertToUnixPath( tmp ) ) );
		// write project directory
		str=SG_SHELLVAR_PROJECTDIR;
		file.WriteImportant(str.arg( YFileOperations::ConvertToUnixPath( projectdir ) ) );
		// write project filename
		str=SG_SHELLVAR_PROJECTFILENAME;
		file.WriteImportant(str.arg( YFileOperations::CorrectFileName( environment->GetName() ) ) );
		// write work directory
		str=SG_SHELLVAR_WORKDIR;
		file.WriteImportant(str.arg( YFileOperations::ConvertToUnixPath( workdir ) ) );
		// write executive name
		str=SG_SHELLVAR_EXECUTIVE;
		file.WriteImportant(str.arg( environment->GetExecutive()->GetName() ) );
		// write executive directory
		str=SG_SHELLVAR_EXECUTIVEDIR;
		YFileOperations::AddSubDirectory(tmp,environment->GetExecutive()->GetPath() );
		file.WriteImportant(str.arg(  YFileOperations::ConvertToUnixPath( tmp ) ) );
		// write logfile name
		str=SG_SHELLVAR_LOGFILE;
		tmp=workdir;
		YFileOperations::AddSubDirectory(tmp,  environment->GetLogFileName() );
		file.WriteImportant(str.arg(YFileOperations::ConvertToUnixPath(tmp) ) );
		// write YASA_SCHEDULERS
		tmp=QString::null;
		for (unsigned int i=0; i<environment->GetSchedulerSet().GetCount(); i++)
		{
			scheduler=((YScheduler*)environment->GetSchedulerSet().At(i))->GetScheduler();
			if ( scheduler && (tmp.find(scheduler->GetShortName()) == -1 ) )
			{
				tmp+=scheduler->GetShortName();
				tmp+=" ";
			}
		}
		str=SG_SHELLVAR_SCHEDULERS;
		if ( tmp.length() )
		{
			str+="\"";
			str+=tmp.left( tmp.length()-1);
			str+="\"";
		}
		else
			str+="\"\"";						// no schedulers
		file.WriteImportant(str);

		// write arg7=compiler options of executive
		str=SG_SHELLVAR_CFLAGS;
		file.WriteImportant(str.arg( environment->GetGCCCompilerOptions().GetContent() ) );
		// write Include flags
		// write C-Files
		tmp=QString::null;
		tmp2=QString::null;
		if ( environment->GetExecutive()->GetSourceCodeSupport() )
		{
			for (unsigned int i=0; i< project->GetSourceSet().GetCount() ; i++)
			{
				source =(YFilePath*)project->GetSourceSet().At(i);
				if ( source && (tmp.find( YFileOperations::ConvertToUnixPath( source->GetAbsPath() ) ) == -1 ) )
				{
					str=source->GetAbsPath();
					YFileOperations::MakeRelative( str, workdir );
					tmp+=str;
					tmp+=" ";
					str=YFileOperations::CutExtension( YFileOperations::GetFileName(str) );
					YFileOperations::AddExtension( str, SG_OBJECT_EXTENSION);
					tmp2+=str;
					tmp2+=" ";
				}
			}
		}
		str=SG_SHELLVAR_CFILES;
		if ( tmp.length() )
		{
			str+="\"";
			str+=tmp.left( tmp.length()-1);
			str+="\"";
		}
		else
			str+="\"\"";						// no source files
		file.WriteImportant(str);
		str=SG_SHELLVAR_OFILES;
		if ( tmp2.length() )
		{
			str+="\"";
			str+=tmp2.left( tmp2.length()-1);
			str+="\"";
		}
		else
			str+="\"\"";						// no object files
		file.WriteImportant(str);

		// write command
		str=SG_SCRIPT_RUNPROJECT;
		str+=" ";
		str+=command;
		file.WriteImportant(str);

		// write "cat logfile && sleep 10000000 && exit(0)"
		if ( command==SG_MAKE_COMMAND_SIMULATE)
		{
			str=SG_SCRIPT_SHOW_LOGFILE;
			file.WriteImportant(str.arg( environment->GetLogFileName() ) );
		}

	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSourceGenerator::CreateProjectSources()
{
	int			retvalue;

	output->Message( MSGP_E_CHECKING_PARAMETERS );
	if ( ! environment )
		return YASAE_ERROR_IN_ARGUMENTS;

	// create projectconfig header
	output->Message( MSGP_E_CREATING_FILE.arg(SOURCE_PROJECTCONFIG_NAME) );
	YSourceFile		pfile(SOURCE_PROJECTCONFIG_NAME);
	retvalue=pfile.OpenFile();
	if ( ! retvalue )
		retvalue=WriteProjectConfig(pfile);
	if ( retvalue )
		return retvalue;

	// create projectexecutive header
	output->Message( MSGP_E_CREATING_FILE.arg(SOURCE_EXECUTIVE_NAME) );
	YSourceFile		efile(SOURCE_EXECUTIVE_NAME);
	retvalue=efile.OpenFile();
	if ( ! retvalue )
		retvalue=WriteExecutive(efile);
	if ( retvalue )
		return retvalue;

	// create projectschedulers header
	output->Message( MSGP_E_CREATING_FILE.arg(SOURCE_SCHEDULERS_NAME) );
	YSourceFile		sfile(SOURCE_SCHEDULERS_NAME);
	retvalue=sfile.OpenFile();
	if ( ! retvalue )
		retvalue=WriteSchedulers(sfile);
	if ( retvalue )
		return retvalue;

	// create projecttaskset header
	output->Message( MSGP_E_CREATING_FILE.arg(SOURCE_TASKSET_HEADER_NAME) );
	YSourceFile		thfile(SOURCE_TASKSET_HEADER_NAME);
	retvalue=thfile.OpenFile();
	if ( ! retvalue )
		retvalue=WriteTaskSetHeader(thfile);
	if ( retvalue )
		return retvalue;

	// create projecttaskset source
	output->Message( MSGP_E_CREATING_FILE.arg(SOURCE_TASKSET_SOURCE_NAME) );
	YSourceFile		tsfile(SOURCE_TASKSET_SOURCE_NAME);
	retvalue=tsfile.OpenFile();
	if ( ! retvalue )
		retvalue=WriteTaskSetSource(tsfile);
	if ( retvalue )
		return retvalue;

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSourceGenerator::WriteProjectConfig(YSourceFile &file)
{
	//YWorkSpace	*workspace=environment->GetProject()->GetWorkSpace();
	//YProject	*project=environment->GetProject();
	QString		str;
	int			retvalue;

	// write header of file
	retvalue=WriteHeader(file, MSGP_SG_DESCRIPTION_PROJECTCONFIG);
	if ( retvalue )
		return retvalue;

	// write content which are allways the same
	file.WriteUnimportant("#ifndef YASA_PROJECT_CONFIG_INCLUDE");
	file.WriteUnimportant("#define	YASA_PROJECT_CONFIG_INCLUDE");
	file.WriteUnimportant();
	file.WriteUnimportant();
	file.WriteUnimportant(SG_COMMENT_LINE);
	file.WriteUnimportant("#include \"yasa_projectexecutive.h\"");
	file.WriteUnimportant("#include \"yasa_projectschedulers.h\"");
	file.WriteUnimportant("#include \"yasa_projecttaskset.h\"");

	// Finalize header
	file.WriteUnimportant();
	file.WriteUnimportant();
	file.WriteUnimportant("#endif\t\t\t\t\t\t\t\t\t\t\t// ifndef YASA_PROJECT_CONFIG_INCLUDE");

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSourceGenerator::WriteExecutive(YSourceFile &file)
{
	//YWorkSpace	*workspace=environment->GetProject()->GetWorkSpace();
	YProject	*project=environment->GetProject();
	QString		str,name,comment;
	int			retvalue;
	bool		protocolfound;

	// write header of file
	retvalue=WriteHeader(file, MSGP_SG_DESCRIPTION_EXECUTIVE);
	if ( retvalue )
		return retvalue;

	// write content which are allways the same
	file.WriteUnimportant("#ifndef YASA_PROJECT_EXECUTIVE_INCLUDE");
	file.WriteUnimportant("#define	YASA_PROJECT_EXECUTIVE_INCLUDE");
	file.WriteUnimportant();
	file.WriteUnimportant();
	file.WriteUnimportant(SG_COMMENT_LINE);
	file.WriteUnimportant("#include <yasa_executive.h>");
	file.WriteUnimportant();
	file.WriteUnimportant();
	file.WriteUnimportant(SG_COMMENT_LINE);

	// environment settings
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_ENVIRONMENTAL_SETTINGS );
	str="#define YP_MAX_CPU\t\t\t%1";
	file.WriteImportant( str.arg( project->GetMaxCPU().GetContent() ) );
	str="#define YP_LOGGINGSIZE\t\t\t%1";
	file.WriteImportant( str.arg( SG_LOGGING_SIZE ) );
	str="#define YP_ENABLE_LOGGING";
	file.WriteImportant(str);
	str="#define YP_SCHEDULINGTIME\t\t%1";
	if ( environment->GetExecutive() && environment->GetExecutive()->GetSchedulingTimeSupport() )
	{
		file.WriteImportant( str.arg( environment->GetSchedulingTime().GetContentString() ) );
		if ( environment->GetSchedulingTime() > environment->GetTimerTick() )
		{
			output->Warning(MSGP_SG_TIMERTICK_TOO_SHORT);
		}
	}
	file.WriteUnimportant();
	file.WriteUnimportant();

	// support of synchronization protocols
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_SUPPORT_SYNC_PROTOCOLS );
	file.WriteUnimportant(SG_COMMENT_LINE);
	file.WriteUnimportant();


	// find used mutex protocols
	for ( int i=1; i<=YASASP_MAX; i++)										// begin at YASASP_PIP
	{
		protocolfound=false;
		for ( unsigned int j=0; j<project->GetMutexSet().GetCount(); j++)
		{
			if ( ((YMutex*)project->GetMutexSet().At(j) )->GetProtocol().GetContent() == i )
				protocolfound=true;
		}

		GetMutexProtocolInfo(i, name, comment);

		str=SG_COMMENT_START+comment;
		file.WriteUnimportant(str);
		if ( name.length() )
		{
			if ( protocolfound )
				str="%1";
			else
				str=SG_IMPORTANT_COMMENT;						// this is not a "real comment"
			str=str.arg("#define YP_SUPPORT_SYNCHRONIZATION_%1");
			file.WriteImportant(str.arg(name) );
		}
		file.WriteUnimportant();
	}
	file.WriteUnimportant();
	file.WriteUnimportant();


	// enable logfile support
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_SUPPORT_LOGGING);
	file.WriteUnimportant("#include <yasa/yasa_loggingsupport.h>");
	file.WriteUnimportant();
	file.WriteUnimportant();

	// Finalize header
	file.WriteUnimportant();
	file.WriteUnimportant();
	file.WriteUnimportant("#endif\t\t\t\t\t\t\t\t\t\t\t// ifndef YASA_PROJECT_EXECUTIVE_INCLUDE");

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSourceGenerator::WriteSchedulers(YSourceFile &file)
{
	//YWorkSpace	*workspace=environment->GetProject()->GetWorkSpace();
	//YProject	*project=environment->GetProject();
	QString		str;
	int			retvalue;

	// write header of file
	retvalue=WriteHeader(file, MSGP_SG_DESCRIPTION_SCHEDULERS);
	if ( retvalue )
		return retvalue;

	// write content which are allways the same
	file.WriteUnimportant("#ifndef YASA_PROJECT_SCHEDULERS_INCLUDE");
	file.WriteUnimportant("#define	YASA_PROJECT_SCHEDULERS_INCLUDE");

	// Finalize header
	file.WriteUnimportant();
	file.WriteUnimportant();
	file.WriteUnimportant("#endif\t\t\t\t\t\t\t\t\t\t\t// ifndef YASA_PROJECT_SCHEDULERS_INCLUDE");

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSourceGenerator::WriteTaskSetHeader(YSourceFile &file)
{
	//YWorkSpace	*workspace=environment->GetProject()->GetWorkSpace();
	YProject	*project=environment->GetProject();
	QString		str,param;
	int			retvalue;

	// write header of file
	retvalue=WriteHeader(file, MSGP_SG_DESCRIPTION_TASKSET_H);
	if ( retvalue )
		return retvalue;

	// write content which are allways the same
	file.WriteUnimportant("#ifndef YASA_PROJECT_TASKSET_INCLUDE");
	file.WriteUnimportant("#define	YASA_PROJECT_TASKSET_INCLUDE");
	file.WriteUnimportant();
	file.WriteUnimportant();

	// task set settings
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteImportant(str+MSGP_SG_PROJECT_SETTINGS );
	str="#define YP_SYNCHRONIZATION_OFFSET\t\t%1";
	file.WriteImportant( str.arg( YTIME::GetSourceString( environment->GetSynchronizationOffset() ) ) );
	str="#define YP_ENDTIME\t\t\t%1";
	file.WriteImportant( str.arg( YTIME::GetSourceString( project->GetEndTime() ) ) );
	if ( ! project->GetEndTime().GetContent() )
		output->Warning(MSGP_SG_ENDTIME_IS_ZERO.arg(project->GetName() ) );

	//#define MSGP_SG_TICK_IS_INFINITY			"Timer tick is set to infinity and calc preemption is set to zero

//	str="#define YP_TICK\t\t\t\t%1";
	/*if ( environment->GetTimerTick() == YASA_TIME_INFINITY )
	{
		output->Warning(MSGP_SG_TICK_IS_INFINITY.arg(project->GetName() ) );
		file.WriteImportant( str.arg( "YASA_DEF_TICK" ) );
	}
	else*/
//	file.WriteImportant( str.arg( YTIME::GetSourceString( environment->GetTimerTick() ) ) );


/*	str="#define YP_CALCPREEMPTIONTIME\t\t%1";
	file.WriteImportant( str.arg( environment->GetCalcPreemptionTime().GetContentString() ) );
	file.WriteUnimportant();
	file.WriteUnimportant();
*/
	// write "#define YP_THREAD_ATTR			thread1" used in startup codes
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_ATTRIBUT_MAPPING );
	str="#define YP_ENVIRONMENT_ATTR\t\t%1";
	file.WriteImportant(str.arg(SG_DEFAULT_ENVIRONMENTATTR_NAME) );
	if ( environment->GetMaxCPU() )
	{
		str="#define YP_SCHEDULING_ATTR\t\t%1";
		file.WriteImportant(str.arg(SG_DEFAULT_SCHEDULERATTR_NAME) );
	}
	if ( project->GetTaskSet().GetCount() )
	{
		str="#define YP_THREAD_ATTR\t\t\t%1";
		if ( project->GetTaskSet().GetCount() )
			param=QString(SG_DEFAULT_THREADATTR_NAME)+"1";
		else
			param="0";
		file.WriteImportant(str.arg(param) );
	}
	if ( project->GetMutexSet().GetCount() )
	{
		str="#define YP_MUTEX_ATTR\t\t\t%1";
		if ( project->GetMutexSet().GetCount() )
			param=QString(SG_DEFAULT_MUTEXATTR_NAME)+"1";
		else
			param="0";
		file.WriteImportant(str.arg(param) );
	}
	file.WriteUnimportant();
	file.WriteUnimportant();


	// write mutex id's
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_MUTEX_SETTINGS );
	for ( unsigned int index=0; index<project->GetMutexSet().GetCount() ; index++)
	{
		str="#define ";
		str+=SG_DEFAULT_MUTEXID_NAME;
		str+="_%1";
		param=project->GetMutexSet().At(index)->GetName();
		ConvertToValidLabelName(param);
		str=str.arg(param);
		str+="\t\t%1";
		file.WriteImportant( str.arg(index+1) );
	}
	file.WriteUnimportant();
	file.WriteUnimportant();


	// Finalize header
	file.WriteUnimportant("#endif\t\t\t\t\t\t\t\t\t\t\t// ifndef YASA_PROJECT_TASKSET_INCLUDE");

	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! workspace, project, executive must be valid !
int YSourceGenerator::WriteTaskSetSource(YSourceFile &file)
{
	//YWorkSpace			*workspace=environment->GetProject()->GetWorkSpace();
	YProject			*project=environment->GetProject();
	YScheduler			*scheduler;
	YTask				*task;
	YMutex				*mutex;
	YResourceAction		*action;
	YExecutivePlugIn	*executive=environment->GetExecutive();
	QString				str,param, name;
	int					retvalue;
	int					index;			// that must be int (not unsigned!)
	int					tmp;
	YASA_TIME			time, lasttime=YTIME::YASA_TIME_MIN;
	YTaskSet			*taskset=(YTaskSet*) &environment->GetProject()->GetTaskSet();

	// write header of file
	retvalue=WriteHeader(file, MSGP_SG_DESCRIPTION_TASKSET_S);
	if ( retvalue )
		return retvalue;

	// write content which are allways the same like "#include <blahblah"
	file.WriteUnimportant("#include <yasa/yasa.h>");
	file.WriteUnimportant();
	file.WriteUnimportant();

	// write scheduler attribute declarations "const extern YASA_SCHEDULER_ATTR dms_attr;"
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteImportant(str+MSGP_SG_PROJECT_SETTINGS );

	for ( index=0; index<environment->GetMaxCPU() ; index++)
	{
		if ( index < (int)environment->GetSchedulerSet().GetCount() )
		{
			scheduler=(YScheduler*)environment->GetSchedulerSet().At(index);
			if ( scheduler )
			{
				if ( scheduler->GetScheduler() )
				{
					str="const extern YASA_SCHEDULER_ATTR %1_attr;";
					file.WriteImportant( str.arg(scheduler->GetScheduler()->GetShortName() ) );
				}
				else
				{
					output->Error(MSGP_EE_SCHEDULER_NOT_DEFINED.arg(index+1) );
					retvalue=YASAE_ERROR_IN_ARGUMENTS;
				}
			}
		}
		else
		{
			output->Error(MSGP_EE_SCHEDULER_NOT_DEFINED.arg(index+1) );
			retvalue=YASAE_ERROR_IN_ARGUMENTS;
		}
		if (retvalue)
			return retvalue;
	}
	file.WriteUnimportant();
	file.WriteUnimportant();

	// write external function declarations "extern void* myinitroutine(void* param, void **userdata);"
	if ( executive->GetSourceCodeSupport() )
	{
		file.WriteUnimportant(SG_COMMENT_LINE);
		str=SG_COMMENT_START;
		file.WriteImportant(str+MSGP_SG_SOURCE_DECLARATIONS );
		for ( index=0; index< (int)project->GetTaskSet().GetCount(); index++)
		{
			task=(YTask*) project->GetTaskSet().At(index);
			if ( task )
			{	// declare Init-Funktion
				if ( task->GetInitFunction().GetContent().length() )
				{
					str=SG_FUNCTION_INIT;
					str+="\t";
					str+=SG_COMMENT_START;
					str+=MSGP_SG_COMMENT_FUNCTION_INIT;
					file.WriteImportant(str.arg( task->GetInitFunction().GetContent() ) );
				}
				// declare Execute-Funktion
				if ( task->GetExecuteFunction().GetContent().length() )
				{
					str=SG_FUNCTION_EXECUTE;
					str+="\t";
					str+=SG_COMMENT_START;
					str+=MSGP_SG_COMMENT_FUNCTION_EXECUTE;
					file.WriteImportant(str.arg( task->GetExecuteFunction().GetContent() ) );
				}
					// declare Cleanup-Funktion
				if ( task->GetCleanupFunction().GetContent().length() )
				{
					str=SG_FUNCTION_CLEANUP;
					str+="\t";
					str+=SG_COMMENT_START;
					str+=MSGP_SG_COMMENT_FUNCTION_CLEANUP;
					file.WriteImportant(str.arg( task->GetCleanupFunction().GetContent() ) );
				}
			}
		}
		file.WriteUnimportant();
		file.WriteUnimportant();
	}


	// write mutex creation array
	for ( index=project->GetMutexSet().GetCount()-1; index>=0 ; --index)
	{	// write comment "Creation array of mutex 'Taskstruktur'"
		mutex=(YMutex*) project->GetMutexSet().At(index);
		file.WriteUnimportant(SG_COMMENT_LINE);
		str=SG_COMMENT_START+MSGP_SG_MUTEX_ARRAY;
		file.WriteUnimportant(str.arg( mutex->GetName() ) );

		// write name of "mutexattr1" struct
		str="YASA_MUTEX_ATTR	";
		str+=SG_DEFAULT_MUTEXATTR_NAME;
		str+="%1=";
		file.WriteImportant( str.arg(index+1) );
		file.WriteImportant( SG_BRACE_OPENED );

		// write next mutex ptr
		if ( index<(int)project->GetMutexSet().GetCount()-1 )
		{
			str="\t&";
			str+=SG_DEFAULT_MUTEXATTR_NAME;
			str+="%1,\t\t\t";
			str+=SG_COMMENT_START;
			str+=MSGP_SG_COMMENT_NEXT_MUTEX;
			file.WriteImportant( str.arg(index+2) );
		}
		else
		{
			str="\t0,\t\t\t\t";
			str+=SG_COMMENT_START;
			str+=MSGP_SG_COMMENT_NEXT_MUTEX;
			file.WriteImportant(str);
		}
		// write YASA_ID of mutex, begin with id
		str="\t{";
		file.WriteUnimportant( str );
		str="\t%1,\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_MUTEXID;			// id of mutex";
		param=SG_DEFAULT_MUTEXID_NAME;
		param+="_%1";
		param=param.arg( mutex->GetName());
		ConvertToValidLabelName(param);
		file.WriteImportant( str.arg(param) );
		// write name
		str="\t\"%1\"\t\t";
		str+=SG_COMMENT_START;
		str+=MSPG_SG_COMMENT_MUTEXNAME;					// name of mutex";
		file.WriteImportant( str.arg( ConvertToValidYasaName( mutex->GetName() ) ) );
		str="\t},";
		file.WriteUnimportant( str );
		// write protocol
		GetMutexProtocolInfo( mutex->GetProtocol(),name,param);
		if ( name.length() )
		{
			str="\tYASASP_%1,\t\t\t";
			str+=SG_COMMENT_START;
			str+=MSGP_SG_COMMENT_SYNC_PROTOCOL;	// synchronization protocol: %1";
			str=str.arg(name);
		}
		else
		{
			str="\t0,\t\t\t\t";
			str+=SG_COMMENT_START;
			str+=MSGP_SG_COMMENT_SYNC_PROTOCOL;	// synchronization protocol: %1";
		}
		file.WriteImportant( str.arg( param ) );
		// write priority ceiling
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_PRIORITYCEILING;	// priority ceiling";
		tmp=mutex->GetPriorityCeiling().GetContent();
		//if (!tmp)
		if ( ! mutex->IsUsed() )
			output->Warning(MSGP_SG_MUTEX_NOT_USED.arg( mutex->GetName() ) );
		file.WriteImportant( str.arg(tmp) );
		// write preempt ceiling
		file.WriteUnimportant( "\t{" );
		str="\t  %1\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_PREEMPTCEILING;	// preempt ceiling";
		file.WriteImportant( str.arg( 0 ) );
		file.WriteUnimportant( "\t}" );
		file.WriteImportant( QString(SG_BRACE_CLOSED)+SG_SEMICOLON);
		file.WriteUnimportant();
	}
	file.WriteUnimportant();
	file.WriteUnimportant();


	// write task creation array
	for ( index=project->GetTaskSet().GetCount()-1; index>=0 ; --index)
	{	task=(YTask*) project->GetTaskSet().At(index);

		// write resume times if defined
		if ( executive->GetResumeTimeSupport() && task->GetResumeTimeSet().GetCount() )
		{
			file.WriteUnimportant(SG_COMMENT_LINE);
			str=SG_COMMENT_START;
			file.WriteUnimportant(str+MSGP_SG_TASK_RESUME_ARRAY.arg( task->GetName() ) );

			str="YASA_TIME ";
			str+=SG_DEFAULT_THREADATTR_NAME;
			str+="%1";
			str=str.arg( index+1 );
			str+=SG_DEFAULT_RESUMETIME_NAME;
			str+="[]=";
			file.WriteImportant( str );
			file.WriteImportant( SG_BRACE_OPENED );

			for ( unsigned int j=0; j<task->GetResumeTimeSet().GetCount() ; j++)
			{
				time=((YResumeTime*)task->GetResumeTimeSet().At(j))->GetTime();
				if ( time )												// skip start times at 0 ! This will be done by offset=0 in thread struct
				{
					if ( lasttime < time )
					{
						lasttime = time;
						str="\t%1,";
						file.WriteImportant( str.arg( YTIME::GetSourceString(time) ) );
					}
					else
					{
						str=MSGP_SG_RESUME_TIME_DECREASED.arg( YTIME::GetSourceString(time) );
						output->Warning( str.arg(task->GetName() ) );
					}
				}

			}
			file.WriteImportant( "\tYASA_TIME_INFINITY" );
			str=SG_BRACE_CLOSED;
			file.WriteImportant( str+SG_SEMICOLON );
		}

		// write resource times if defined
		if ( executive->GetResourceActionsSupport() && task->GetResourceActionSet().GetCount() )
		{
			// write resource times comment with task name
			file.WriteUnimportant(SG_COMMENT_LINE);
			str=SG_COMMENT_START;
			file.WriteUnimportant(str+MSGP_SG_TASK_RESOURCE_ARRAY.arg( task->GetName()) );

			str="YASA_RESOURCEVECTOR ";
			str+=SG_DEFAULT_THREADATTR_NAME;
			str+="%1";
			str=str.arg( index+1 );
			str+=SG_DEFAULT_RESOURCEACTION_NAME;
			str+="[]=";
			file.WriteImportant( str );
			file.WriteImportant( SG_BRACE_OPENED );

			for ( unsigned int j=0; j<task->GetResourceActionSet().GetCount() ; j++)
			{	// create: "{ (void*)MUTEXID_1, YASAA_MUTEX_LOCK  ,					  0 }, "
				action=(YResourceAction*)task->GetResourceActionSet().At(j);
				if ( action->GetMutex() )
				{	// write MutexID ( This is a cast to (void*) !)
					str="\t{\t(void*)%1, ";
					param=SG_DEFAULT_MUTEXID_NAME;
					param+="_%1";
					// write mutex name to lock/unlock
					param=param.arg(action->GetMutex()->GetName() );
					str=str.arg( ConvertToValidLabelName(param) );
					GetResourceActionInfo(action->GetAction(), param);
					str+=param;
					str+="\t, ";
					// write time of resource action
					if (  action->GetTime() > task->GetRequiredTime() )
					{
						output->Warning( MSGP_SG_ACTIONTIME_TO_HIGH.arg(action->GetMutex()->GetName() )
																	.arg(task->GetName() )
																	.arg(action->GetTime().GetVisibleContentString() )
																	.arg(task->GetRequiredTime().GetVisibleContentString() ) );
						str+=  YTIME::GetSourceString( task->GetRequiredTime() );
					}
					else
						str+=  YTIME::GetSourceString( action->GetTime() );
					str+="\t},";
					file.WriteImportant( str );
				}
				else
				{	// mutex not defined in resource action
					param=MSGP_SG_MUTEX_NOT_DEFINED.arg( task->GetName() );
					output->Warning( param.arg(action->GetTime().GetContentString() ) );
					param="0";
				}

			}
			file.WriteImportant( "\t{\t0,\t0,\tYASA_TIME_INFINITY\t}" );
			file.WriteImportant( QString(SG_BRACE_CLOSED)+SG_SEMICOLON );
			file.WriteUnimportant();
		}

		// write thread struct
		file.WriteUnimportant(SG_COMMENT_LINE);
		str=SG_COMMENT_START;
		file.WriteImportant(str+MSGP_SG_TASK_ARRAY.arg( task->GetName() ) );
		// write head of structure "YASA_THREAD_ATTR	threadxx= "
		str="YASA_THREAD_ATTR ";
		str+=SG_DEFAULT_THREADATTR_NAME;
		str+="%1=";
		file.WriteImportant( str.arg(index+1) );
		file.WriteImportant( SG_BRACE_OPENED );
		// write next thread ptr
		if ( index<(int)project->GetTaskSet().GetCount()-1 )
		{
			str="\t&";
			str+=SG_DEFAULT_THREADATTR_NAME;
			str+="%1,\t\t\t";
			str+=SG_COMMENT_START;
			str+=MSGP_SG_COMMENT_NEXT_THREAD;
			file.WriteImportant( str.arg(index+2) );
		}
		else
		{
			str="\t0,\t\t\t\t";
			str+=SG_COMMENT_START;
			str+=MSGP_SG_COMMENT_NEXT_THREAD;
			file.WriteImportant( str );
		}
		// write "YASA_ID" structure
		str="\t";
		str+=SG_BRACE_OPENED;
		file.WriteImportant( str );
		str=QString("\t  %1,\t\t\t\t").arg(index+1);										// write task id
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_THREADID;
		file.WriteImportant( str );
		str=QString("\t  \"%1\"\t\t\t").arg( ConvertToValidYasaName(task->GetName() ) );	// write task name
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_THREADNAME;
		file.WriteImportant( str );
		str="\t";
		str+=SG_BRACE_CLOSED;
		str+=",";
		file.WriteImportant( str );
		// write thread type
		str="\t%1,\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_THREADTYPE;
		file.WriteImportant( str.arg( GetTaskTypeInfo(task) ) );

		// write priority
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_PRIORITY;
		file.WriteImportant( str.arg(task->GetPriority().GetContentString() ) );

		// write period
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_PERIOD;
		file.WriteImportant( str.arg(  YTIME::GetSourceString(task->GetPeriod() ) ) );
		if ( task->GetPeriod() < environment->GetTimerTick() )
			output->Warning(MSGP_SG_PERIOD_SMALLER_THAN_TICK
							.arg( task->GetName() )
							.arg( task->GetPeriod().GetVisibleContentString() )
							.arg( environment->GetTimerTick().GetVisibleContentString() ) );
		if ( task->GetPeriod().GetContent() && ! task->GetPeriod().IsInfinity() && task->GetTaskType()!=YASA_THREAD_PERIODIC )
			output->Warning( MSGP_SG_PERIOD_BUT_NOT_PERIODIC.arg( task->GetName() ).arg( task->GetPeriod().GetVisibleContentString() ) );

		// write Required time
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_REQUIRED_TIME;
		file.WriteImportant( str.arg( YTIME::GetSourceString(task->GetRequiredTime() ) ) );
		if ( task->GetPeriod() < task->GetRequiredTime() )
			output->Warning(MSGP_SG_REQUIREDTIME_HIGHER_THAN_PERIOD
							.arg( task->GetName() )
							.arg( task->GetRequiredTime().GetVisibleContentString() )
							.arg( task->GetPeriod().GetVisibleContentString() ) );
		if ( task->GetDeadline() < task->GetRequiredTime() )
			output->Warning(MSGP_SG_REQUIREDTIME_HIGHER_THAN_DEADLINE
							.arg( task->GetName() )
							.arg( task->GetRequiredTime().GetVisibleContentString() )
							.arg( task->GetDeadline().GetVisibleContentString() ) );

		// write Deadline time
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_DEADLINE;
		file.WriteImportant( str.arg(  YTIME::GetSourceString(task->GetDeadline() ) ) );
		if ( task->GetPeriod() < task->GetDeadline() )
			output->Warning(MSGP_SG_DEADLINE_HIGHER_THAN_PERIOD
							.arg( task->GetName() )
							.arg( task->GetDeadline().GetVisibleContentString() )
							.arg( task->GetPeriod().GetVisibleContentString() ) );

		// write Deadline tolerance time
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_DEADLINE_TOLERANCE;
		file.WriteImportant( str.arg(  YTIME::GetSourceString(task->GetDeadlineTolerance() ) ) );

		// write Offset time
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_OFFSET;
		file.WriteImportant( str.arg(  YTIME::GetSourceString(task->GetOffset() ) ) );

		// write resume vector ptr
		if ( executive->GetResumeTimeSupport() && task->GetResumeTimeSet().GetCount() )
		{
			str="\t";
			str+=SG_DEFAULT_THREADATTR_NAME;
			str+="%1";
			str=str.arg( index+1 );
			str+=SG_DEFAULT_RESUMETIME_NAME;
			str+=",\t\t";
		}
		else
			str="\t0,\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_RESUME_VECTOR;
		file.WriteImportant( str );
		// write resource action ptr "&threadattr4resourceactions,	// Resource vector"
		if ( executive->GetResourceActionsSupport() && task->GetResourceActionSet().GetCount() )
		{
			str="\t";
			str+=SG_DEFAULT_THREADATTR_NAME;
			str+="%1";
			str=str.arg( index+1 );
			str+=SG_DEFAULT_RESOURCEACTION_NAME;
			str+=",\t";
		}
		else
			str="\t0,\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_RESOURCE_VECTOR;

		file.WriteImportant( str );
		// write  cpu
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_CPU;
		tmp=task->GetCPU().GetContent();
		if ( ! executive->GetChangeableCPUSupport() && tmp==0 )
		{
			output->Warning(MSGP_SG_CHANGEABLE_CPU_NOT_SUPPORTED.arg( task->GetName() ) );
			tmp=YG_DEFAULT_THREAD_CPU;
		}
		tmp--;
		file.WriteImportant( str.arg(tmp) );

		// write deadline behaviour
		str="\t%1,\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_DEADLINE_BEHAVIOUR;
		file.WriteImportant( str.arg( GetDeadlineBehaviour(task) ) );

		// write emergency thread
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_EMERGENCY_THREAD;
		param="0";
		if ( task->GetEmergencyThread() )
		{
			param=SG_DEFAULT_THREADATTR_NAME;
			param+="%1";
			param=param.arg( taskset->FindItemIndex( task->GetEmergencyThread() ) );
		}
		file.WriteImportant( str.arg(param) );

		// init function
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_FUNCTION_INIT;
		param="0";
		if ( environment->GetExecutive()->GetSourceCodeSupport() )
		{
			if ( task->GetInitFunction().GetContent().length() )
				param=task->GetInitFunction().GetContent();
			else
				output->Warning( MSGP_SG_INIT_FUNCTION_MISSING.arg( task->GetName() ) );
		}
		file.WriteImportant( str.arg(param) );

		// execute function
		str="\t%1,\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_FUNCTION_EXECUTE;
		param="0";
		if ( environment->GetExecutive()->GetSourceCodeSupport() )
		{
			if ( task->GetExecuteFunction().GetContent().length() )
				param=task->GetExecuteFunction().GetContent();
			else
				output->Warning( MSGP_SG_EXECUTE_FUNCTION_MISSING.arg( task->GetName() ) );
		}
		file.WriteImportant( str.arg(param) );

		// cleanup function
		str="\t%1\t\t\t\t";
		str+=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_FUNCTION_CLEANUP;
		param="0";
		if ( environment->GetExecutive()->GetSourceCodeSupport() )
		{
			if ( task->GetCleanupFunction().GetContent().length() )
				param=task->GetCleanupFunction().GetContent();
			else
				output->Warning( MSGP_SG_CLEANUP_FUNCTION_MISSING.arg( task->GetName() ) );
		}
		file.WriteImportant( str.arg(param) );

		// Finalize task
		file.WriteImportant( QString(SG_BRACE_CLOSED)+SG_SEMICOLON);
		file.WriteUnimportant();
		file.WriteUnimportant();
		file.WriteUnimportant();
	}

	// write scheduler attributes
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_SCHEDULER_ARRAY );
	str=QString("const YASA_SCHEDULING_ATTR %1[YP_MAX_CPU]=").arg(SG_DEFAULT_SCHEDULERATTR_NAME);
	file.WriteUnimportant(str);
	file.WriteUnimportant("{");
	for ( index=0; index<environment->GetMaxCPU() ; index++)
	{
		if ( (int)environment->GetSchedulerSet().GetCount() > index )
			scheduler=(YScheduler*)environment->GetSchedulerSet().At(index);
		else
			scheduler=0;
		if ( scheduler && scheduler->GetScheduler() )
		{
			str="\t{\t&%1_attr\t, YP_ENDTIME,\t%2\t, %3\t},";
			str=str.arg( scheduler->GetScheduler()->GetShortName() );
			str=str.arg( scheduler->GetTimerTick().GetContentString() );
			str=str.arg( scheduler->GetCalcPreemptionTime().GetContent() );
		}
		else
		{
			str="{\t0, YP_ENDTIME\t, %1\t, %2\t}, ";
			str=str.arg( environment->GetTimerTick().GetContentString() );
			str=str.arg( environment->GetCalcPreemptionTime().GetContent() );
		}
		file.WriteImportant(str);

	}
	file.WriteUnimportant("};");
	file.WriteUnimportant();
	file.WriteUnimportant();



	// write environment attributes
	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str+MSGP_SG_ENVIRONMENT_ATTR );
	str=QString("YASA_ENVIRONMENT_ATTR %1=").arg(SG_DEFAULT_ENVIRONMENTATTR_NAME);
	file.WriteUnimportant(str);
	file.WriteUnimportant("{");

	file.WriteImportant(QString("\tYP_SCHEDULING_ATTR,\t\t\t// %1").arg(MSGP_SG_COMMENT_SCHEDULING_ATTR) );
	file.WriteImportant("#ifdef YP_THREAD_ATTR");
	file.WriteImportant(QString("\t&YP_THREAD_ATTR,\t\t\t// %1"). arg(MSGP_SG_COMMENT_THREAD_ATTR) );
	file.WriteImportant("#else");
	file.WriteImportant("\t0,");
	file.WriteImportant("#endif");
	file.WriteImportant("#ifdef YP_MUTEX_ATTR");
	file.WriteImportant(QString("\t&YP_MUTEX_ATTR,\t\t\t// %1").arg(MSGP_SG_COMMENT_MUTEX_ATTR) );
	file.WriteImportant("#else");
	file.WriteImportant("\t0,");
	file.WriteImportant("#endif");
	file.WriteImportant(QString("\tYP_SYNCHRONIZATION_OFFSET,\t\t// %1").arg(MSGP_SG_COMMENT_SYNCHRONIZATION_OFFSET) );
	file.WriteUnimportant("};");








	// Finalize source
	file.WriteUnimportant();
	file.WriteUnimportant();

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Writes the top of the source/header such as description of file, author,
int YSourceGenerator::WriteHeader(YSourceFile &file, const QString &description)
{
	YWorkSpace	*workspace=environment->GetProject()->GetWorkSpace();
	YProject	*project=environment->GetProject();
	QString		str;
	bool		descriptionprinted=false;

	file.WriteUnimportant(SG_COMMENT_LINE);
	str=SG_COMMENT_START;
	file.WriteUnimportant(str);

	// Write description
	if ( description.length() )
	{
		str=SG_COMMENT_START;
		str+=description;
		file.WriteUnimportant(str);
		str=SG_COMMENT_START;
		file.WriteUnimportant(str);
	}

	// write project name
	str=SG_COMMENT_START;
	str+=MSGP_SG_PROJECT.arg(project->GetName());
	file.WriteUnimportant(str);

	// write author name
	str=SG_COMMENT_START;
	str+=MSGP_SG_AUTHOR.arg(workspace->GetAuthor() );
	file.WriteUnimportant(str);

	// write file name
	str=SG_COMMENT_START;
	str+=MSGP_SG_FILENAME.arg(environment->GetName());
	str=str.arg( file.name() );
	file.WriteUnimportant(str);

	// write creation date
	str=SG_COMMENT_START;
	str+=MSGP_SG_CREATION_DATE.arg(workspace->GetCreationTime().toString() );
	file.WriteUnimportant(str);

	// write last modified date
	str=SG_COMMENT_START;
	str+=MSGP_SG_LASTMODIFIED_DATE.arg(workspace->GetLastChangedTime().toString() );
	file.WriteUnimportant(str);

	// finalize top of header
	str=SG_COMMENT_START;
	file.WriteUnimportant(str);
	file.WriteUnimportant(SG_COMMENT_LINE);

	// write workspace description
	if ( workspace->GetDescription().length() )
	{
		file.WriteUnimportant(SG_COMMENT_START);
		str=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_WORKSPACE_DESCRIPTION;
		file.WriteUnimportant(str);
		str=SG_COMMENT_START;
		str+=workspace->GetDescription();
		file.WriteUnimportant(str);
		file.WriteUnimportant(SG_COMMENT_START);
		descriptionprinted=true;
	}

	// write project description
	if ( project->GetDescription().length() )
	{
		file.WriteUnimportant(SG_COMMENT_START);
		str=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_PROJECT_DESCRIPTION;
		file.WriteUnimportant(str);
		str=SG_COMMENT_START;
		str+=project->GetDescription();
		file.WriteUnimportant(str);
		file.WriteUnimportant(SG_COMMENT_START);
		descriptionprinted=true;
	}

	// write environment description
	if ( environment->GetDescription().length() )
	{
		file.WriteUnimportant(SG_COMMENT_START);
		str=SG_COMMENT_START;
		str+=MSGP_SG_COMMENT_ENVIRONMENT_DESCRIPTION;
		file.WriteUnimportant(str);
		str=SG_COMMENT_START;
		str+=environment->GetDescription();
		file.WriteUnimportant(str);
		file.WriteUnimportant(SG_COMMENT_START);
		descriptionprinted=true;
	}

	//
	if ( descriptionprinted )
	{
		file.WriteUnimportant(SG_COMMENT_LINE);
	}

	file.WriteUnimportant(SG_COMMENT_START);
	file.WriteUnimportant(MSGP_SG_GPL_LICENSE_C_SOURCE);
	file.WriteUnimportant(SG_COMMENT_START);
	file.WriteUnimportant(SG_COMMENT_LINE);
	file.WriteUnimportant();

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString& YSourceGenerator::ConvertToValidLabelName(QString &name)
{
	QChar	a,b;
	for (unsigned int i=0 ; i<name.length(); i++)
	{
		a=name.at(i);	//[i];
		if ( !(a>='A' && a<='Z') && !(a>='0' && a<='9') && a!='_' )
		{
			b='_';
			if ( a>='a' && a<='z' )
				b=a-0x20;							// Make upper case
			name.replace(i, 1, &b, 1);
		}
	}
	return name;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Yasa supports only limited names of tasks. So there will be cutted here
//! This is only needed using YASA_ID
QString	YSourceGenerator::ConvertToValidYasaName(const QString &name)
{
	if ( name.length() <= YASA_MAXNAME )
		return name;
	else
	{
		output->Warning(MSGP_SG_NAME_TO_LONG.arg(name).arg(name.left(YASA_MAXNAME) ) );
		return name.left(YASA_MAXNAME);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSourceGenerator::GetMutexProtocolInfo(int protocol, QString &name, QString &comment)
{
	switch ( protocol )
	{
	case YASASP_NOTHING:comment=MSGP_SG_COMMENT_SYNC_NOTHING;
						name="NOTHING";
						break;
	case YASASP_PIP:	comment=MSGP_SG_COMMENT_SYNC_PIP;
						name="PIP";
						break;
	case YASASP_CSP:	comment=MSGP_SG_COMMENT_SYNC_CSP;
						name="CSP";
						break;
	case YASASP_PCP:	comment=MSGP_SG_COMMENT_SYNC_PCP;
						name="PCP";
						break;
	case YASASP_DPCP:	comment=MSGP_SG_COMMENT_SYNC_DPCP;
						name="DPCP";
						break;
	case YASASP_KP:		comment=MSGP_SG_COMMENT_SYNC_KP;
						name="KP";
						break;
	case YASASP_SRP:	comment=MSGP_SG_COMMENT_SYNC_SRP;
						name="SRP";
						break;
	default:			name=QString::null;
						comment=QString(MSGP_SG_SUPPORT_SYNC_PROTOCOL_SKIPPED).arg(protocol);
						output->Warning(comment);
						break;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YSourceGenerator::GetResourceActionInfo(int action, QString &param)
{
	switch (action)
	{
	case YASAA_MUTEX_LOCK:	param="YASAA_MUTEX_LOCK  ";
							break;
	case YASAA_MUTEX_UNLOCK:param="YASAA_MUTEX_UNLOCK";
							break;
	default:				param=QString::null;
							output->Warning( QString( MSGP_SG_ACTION_SKIPPED).arg(action) );
							break;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YSourceGenerator::GetTaskTypeInfo(YTask *task)
{
	switch (task->GetTaskType().GetContent() )
	{
	default:				output->Warning( QString( MSGP_SG_TASKTYPE_NOT_DEFINED).arg(task->GetName() ) );
	case YASA_THREAD_NONPERIODIC:
							return QString("YASA_THREAD_NONPERIODIC");
	case YASA_THREAD_PERIODIC:
							if ( ! task->GetPeriod().IsInfinity() )
								return QString("YASA_THREAD_PERIODIC");
							else
							{
								output->Warning( MSGP_SG_PERIOD_INFINITY_TYPE_PERIODIC.arg( task->GetName() ) );
								return QString("YASA_THREAD_NONPERIODIC");
							}
	case YASA_THREAD_ASYNCHRONOUS:
							return QString("YASA_THREAD_ASYNCHRONOUS");
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YSourceGenerator::GetDeadlineBehaviour(YTask *task)
{
	switch (task->GetDeadlineBehaviour().GetContent() )
	{
	default:				output->Warning( QString( MSGP_SG_DEADLINEBEHAVIOUR_NOT_DEFINED).arg(task->GetName() ) );
	case YASA_DL_IGNORE:
							return QString("YASA_DL_IGNORE");
	case YASA_DL_CANCEL_THREAD:
							if ( environment->GetExecutive()->GetEmergencyThreadSupport() )
								return QString("YASA_DL_CANCEL_THREAD");
							else
							{
								QString str=MSGP_SG_CANCEL_THREADS_NOT_SUPPORTED.arg(task->GetName());
								output->Warning( str.arg(environment->GetExecutive()->GetName() ) );
								return QString("YASA_DL_IGNORE");
							}
	case YASA_DL_SKIPOVER:
							return QString("YASA_DL_SKIPOVER");
	case YASA_DL_EMERGENCYTHREAD:
							if ( task->GetEmergencyThread() )
							{
								if ( environment->GetExecutive()->GetEmergencyThreadSupport() )
									return QString("YASA_DL_EMERGENCYTHREAD");
								else
								{
									QString str=MSGP_SG_EMERGENCY_TASK_NOT_SUPPORTED.arg(task->GetName());
									output->Warning( str.arg(environment->GetExecutive()->GetName() ) );
									return QString("YASA_DL_IGNORE");
								}
							}
							output->Warning( MSGP_SG_EMERGENCY_TASK_NOT_DEFINED.arg(task->GetName() ) );
							return QString("YASA_DL_IGNORE");
	}
}
