//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Timing handler code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/03/18
// $Header: /sources/yasa/yasagui/timinghandler.cpp,v 1.4 2003/01/23 09:18:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "timinghandler.h"
#include "timings.h"
#include "application.h"
#include "output.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTimingInfo::~YTimingInfo()
{
	if ( timings )
		delete timings;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Insert a timing widget to the current scorllview
//! Attention: The timings instance are created only. In this function timings->InitInstance() will be called!
int YTimingHandler::InsertTiming(YTimings *timings)
{
	QPoint		position;
	YTimingInfo	*ti;
	int			retvalue=YASAE_OUT_OF_MEMORY;

	if ( timings )
	{
		retvalue=timings->InitInstance( parent->font(), &ypainter );
		if ( ! retvalue )
		{
			timings->UpdateInstance( ypainter.GetZoomFactor() );
			ti = new YTimingInfo();
			if ( ti )
			{
				// the timing class will be sorted vertically
				if ( timinginfos.count() )
				{
					position.setY( timinginfos.getLast()->GetPosition().y() + timinginfos.getLast()->GetTimings()->GetHeight() );
				}
				ti->SetPosition(position);
				ti->SetTimings(timings);
				if ( absmaxtime < ti->GetTimings()->GetMaxDuration() )
					absmaxtime = ti->GetTimings()->GetMaxDuration();

				timinginfos.append(ti);
				emit NewTimingInserted(ti);
				retvalue=0;
			}
		}
	}
	return retvalue;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTimingHandler::ZoomIn()
{
	if ( ypainter.GetZoomFactor() > YTIMING_DEFAULT_ZOOMFACTOR_CHANGE )
	{
		ypainter.SetZoomFactor( absmaxtime, ypainter.GetZoomFactor()/YTIMING_DEFAULT_ZOOMFACTOR_CHANGE );
		UpdateTimings();
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTimingHandler::ZoomOut()
{
	if ( ypainter.GetZoomFactor() < absmaxtime/YTIMING_DEFAULT_ZOOMFACTOR_CHANGE/100 )
	{
		ypainter.SetZoomFactor( absmaxtime, ypainter.GetZoomFactor()*YTIMING_DEFAULT_ZOOMFACTOR_CHANGE );
		UpdateTimings();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Redraw all timings
void YTimingHandler::UpdateTimings()
{
	YTimingInfo		*tinfo=timinginfos.first();
	YTimings		*timings;

	while (tinfo)
	{
		timings=tinfo->GetTimings();
		if ( timings )
		{
			timings->UpdateInstance( ypainter.GetZoomFactor() );
		}
		tinfo=timinginfos.next();
	};
	UpdateViews();
	if (parent)
		parent->repaint();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is called to recalculate the width of the scrollviews
void YTimingDiagramScrollView::Resize()
{
	YTimingInfo		*tinfo=GetTimingHandler()->GetTimingInfos()->first();
	size.setWidth(0);
	while (tinfo)
	{
		if ( size.width() < tinfo->GetTimings()->GetWidth() )
			size.setWidth( tinfo->GetTimings()->GetWidth() );
		tinfo=GetTimingHandler()->GetTimingInfos()->next();
	};
	resizeContents( size.width(), size.height() );
	updateContents( 0, 0, size.width(), size.height() );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is called to update the whole timing window, e.g. if zoomfactorx was changed
void YDiagramNameTimingHandler::UpdateViews()
{
	int		oldposy=diagramscrollview.contentsY();
	INT64	oldwidth=diagramscrollview.contentsWidth();
	INT64	oldmiddleposx=diagramscrollview.contentsX()+(diagramscrollview.visibleWidth()+1)/2;
	if ( ! diagramscrollview.contentsX() )
		oldmiddleposx=0;
	namescrollview.Resize();
	diagramscrollview.Resize();
	INT64	newmiddleposx=oldmiddleposx*diagramscrollview.GetSize().width()/oldwidth;
	INT64	newpos=newmiddleposx-(diagramscrollview.visibleWidth()+1)/2;
	if (newpos<0)
		newpos=0;
	diagramscrollview.setContentsPos( (int)newpos, oldposy);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a slot which is called after the scrollview tsv is scrolled. It checks a recursion loop
void YTimingHandler::ViewScrolled(YTimingScrollView *tsv, int , int posy)
{
	// 2 means two splitted windows, only 1 call allowed to other client
	if ( scrollcount< 1 )
	{
		scrollcount++;
		tsv->scrollBy( 0, posy-tsv->contentsY() );
		scrollcount--;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTimingHandler::PostCreate()
{
	//! Check if there are one or more valid timings
	if ( timinginfos.count() )
	{
		ypainter.SetZoomFactor( absmaxtime, 0 );
		return 0;
	}
	else
	{
		YApplication::GetOutput()->Warning(MSGP_TI_WINDOW_CLOSED );
		return YASAE_ERROR_IN_ARGUMENTS;
	}
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTimingScrollView::ViewScrolled(int posx, int posy)
{
	timinghandler->ViewScrolled(this, posx, posy);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTimingDiagramScrollView::NewTimingInserted(YTimingInfo *ti)
{
	size.setHeight( ti->GetPosition().y() + ti->GetTimings()->GetHeight() );
	if ( size.width() < ti->GetTimings()->GetWidth() )
		size.setWidth( ti->GetTimings()->GetWidth() );
	resizeContents( size.width(), size.height() );
}

#include <qframe.h>
#include <qlayout.h>
#include <qtextview.h>
#define TIMING_KEYSCROLLX		20
#define TIMING_KEYSCROLLY		15
#define TIMING_TOOLTIPMOVERANGE	10
void YTimingDiagramScrollView::keyPressEvent ( QKeyEvent *event )
{
	Qt::ButtonState	state=event->state();
	switch ( event->ascii() )
	{
		case '+':			GetTimingHandler()->ZoomIn();
							return;
		case '-':			GetTimingHandler()->ZoomOut();
							return;
		case ' ':			{
		QFrame				*mainwidget = new QFrame( 0, QString::null, WType_TopLevel | WStyle_StaysOnTop |WStyle_NormalBorder | WDestructiveClose | Qt::WStyle_Tool);
		QVBoxLayout			*vboxlayout	= new QVBoxLayout(mainwidget, MDI_SCROLLVIEW_BORDER );
		QTextView			*tv			= new QTextView( mainwidget, "huhuhuhu" );
		vboxlayout->addWidget( tv );
		mainwidget->resize(250,150);
		vboxlayout->setMargin(2);
		mainwidget->move( 300,300);
		tv->setText("so, jetzt's gehts aber los<br>a<br>b<br>c<br>d<br>e<br>f<br>g<br>h<br>i<br>j<br>k<br>jh<br>d\nk\nhsdksd\njshdjshdjshd\nkjdksdjksd\nsdjhsjdhds");
		mainwidget->show();

							}
		default:			break;
	}
	switch ( event->key() )
	{
	// scrolling left
	case Key_Left:			if ( state != Qt::ShiftButton && state != Qt::ControlButton)
							{
								scrollBy( -TIMING_KEYSCROLLX, 0 );
								return;
							}
							if ( state == Qt::ShiftButton )
							{
								scrollBy( -visibleWidth()+TIMING_KEYSCROLLX, 0 );
								return;
							}
	case Key_Home:			scrollBy( -contentsX(), 0 );
							return;

	// scrolling right
	case Key_Right:			if ( state != Qt::ShiftButton && state != Qt::ControlButton)
							{
								scrollBy( TIMING_KEYSCROLLX, 0 );
								return;
							}
							if ( state == Qt::ShiftButton )
							{
								scrollBy( visibleWidth()-TIMING_KEYSCROLLX, 0 );
								return;
							}
	case Key_End:			scrollBy( contentsWidth()-contentsX(), 0 );
							return;

	// scrolling up
	case Key_Up:			if ( state != Qt::ShiftButton && state != Qt::ControlButton)
							{
								scrollBy( 0, -TIMING_KEYSCROLLY );
								return;
							}
	case Key_PageUp:		if ( state == Qt::ControlButton )
							{
								scrollBy( 0, -contentsHeight() );
								return;
							}
							scrollBy( 0, -visibleHeight()+TIMING_KEYSCROLLY );
							return;

	// scrolling down
	case Key_Down:			if ( state != Qt::ShiftButton && state != Qt::ControlButton)
							{
								scrollBy( 0, TIMING_KEYSCROLLY );
								return;
							}
	case Key_PageDown:		if ( state == Qt::ControlButton )
							{
								scrollBy( 0, contentsHeight()-contentsY() );
								return;
							}
							scrollBy( 0, visibleHeight()-TIMING_KEYSCROLLY );
							return;

	default:				event->ignore();
							return;
	}
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is called by the system to redraw a region of the scrollview. Here is an own
//! clipping function used.
void YTimingDiagramScrollView::drawContents(QPainter* painter, int cx, int cy, int cw, int ch)
{
	YPainter		*ypainter=GetTimingHandler()->GetPainter();
	YTimingInfo		*tinfo=GetTimingHandler()->GetTimingInfos()->first();
	int				x,y,xw,yh;
	const YTimings	*timings;

	ypainter->SetPainter(painter);
	painter->fillRect( cx, cy, cw, ch, ypainter->GetWindowBackgroundBrush() );
	while (tinfo)
	{
		timings=tinfo->GetTimings();
		if ( timings )
		{	// check if client rect is in widget rect
			if ( ( (cy>=tinfo->GetPosition().y() ) && (cy   <= tinfo->GetPosition().y()+timings->GetHeight() ) ||
				   (cy<=tinfo->GetPosition().y() ) && (cy+ch>= tinfo->GetPosition().y() )							) &&
				 ( (cx>=tinfo->GetPosition().x() ) && (cx   <= tinfo->GetPosition().x()+timings->GetWidth() ) ||
				   (cx<=tinfo->GetPosition().x() ) && (cx+cw>= tinfo->GetPosition().x() ) )							)
			{
				x=cx-tinfo->GetPosition().x();		// get x position in widget
				y=cy-tinfo->GetPosition().y();		// get y position in widget
				if ( x<0 ) x=0;						// if x<0 widget is scrolled
				if ( y<0 ) y=0;						// if y<0 widget is scrolled
				xw = ( cw > timings->GetWidth() )  ? timings->GetWidth()  : cw;
				yh = ( ch > timings->GetHeight() ) ? timings->GetHeight() : ch;
				ypainter->SetRoot( tinfo->GetPosition() );
				ypainter->SetUpdateArea(x,y,xw,yh);
				timings->DrawTimingContents( *ypainter );
			}
		}
		tinfo=GetTimingHandler()->GetTimingInfos()->next();
	};
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YTimingNameScrollView::NewTimingInserted(YTimingInfo *ti)
{
	size.setHeight( ti->GetPosition().y() + ti->GetTimings()->GetHeight() );
	if ( size.width() < ti->GetTimings()->GetNameWidth() )
		size.setWidth( ti->GetTimings()->GetNameWidth() );
	resizeContents( size.width(), size.height() );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is called by the system to redraw a region of the scrollview. Here is an own
//! clipping function used.
void YTimingNameScrollView::drawContents(QPainter* painter, int cx, int cy, int cw, int ch)
{
	YPainter		*ypainter=GetTimingHandler()->GetPainter();
	YTimingInfo		*tinfo=GetTimingHandler()->GetTimingInfos()->first();
	int				x,y,xw,yh;
	const YTimings	*timings;

	ypainter->SetPainter(painter);
	painter->fillRect( cx, cy, cw, ch, ypainter->GetWindowBackgroundBrush() );
	while (tinfo)
	{
		timings=tinfo->GetTimings();
		if ( timings )
		{
			if ( ( (cy>=tinfo->GetPosition().y() ) && (cy   <= tinfo->GetPosition().y()+timings->GetHeight() ) ||
				   (cy<=tinfo->GetPosition().y() ) && (cy+ch>= tinfo->GetPosition().y() )							) &&
				 ( (cx>=tinfo->GetPosition().x() ) && (cx   <= tinfo->GetPosition().x()+timings->GetWidth() ) ||
				   (cx<=tinfo->GetPosition().x() ) && (cx+cw>= tinfo->GetPosition().x() ) )							)
			{	// the timing names will be printed every time (check saved)
				x=cx-tinfo->GetPosition().x();
				y=cy-tinfo->GetPosition().y();
				if ( x<0 ) x=0;
				if ( y<0 ) y=0;
				xw = ( cw > timings->GetWidth() )  ? timings->GetWidth()  : cw;
				yh = ( ch > timings->GetHeight() ) ? timings->GetHeight() : ch;
				ypainter->SetRoot( tinfo->GetPosition() );
				ypainter->SetUpdateArea(x,y,xw,yh);
				timings->DrawNameContents( *ypainter );
			}
		}
		tinfo=GetTimingHandler()->GetTimingInfos()->next();
	};
}




