//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Timing painter code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/timingpainter.cpp,v 1.5 2003/01/30 12:25:08 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "timingpainter.h"
#include "application.h"
#include "settings.h"
#include "logfile.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YPainter::YPainter() :
painter(0),
logfilepixmaps( ((YApplication*)qApp)->GetSettings()->GetLogFileFigures() ),
windowbackground( *((YApplication*)qApp)->GetSettings()->GetApplicationFigures().GetBrush(YApplicationFigures::BACKGROUND_TIMINGWINDOWS) ),
systemtaskbrush( *logfilepixmaps.GetLogFileBrush( YLogFileFigures::SYSTEM_TASK_BRUSH ) ),
taskbrush( *logfilepixmaps.GetLogFileBrush( YLogFileFigures::TASK_BRUSH ) ),
taskidlebrush( *logfilepixmaps.GetLogFileBrush( YLogFileFigures::TASK_IDLE_BRUSH ) ),
taskexcludedbrush( *logfilepixmaps.GetLogFileBrush( YLogFileFigures::TASK_EXCLUDED_BRUSH ) ),
mutexbrush( *logfilepixmaps.GetLogFileBrush( YLogFileFigures::MUTEX_BRUSH ) ),
jitterbrush( *logfilepixmaps.GetLogFileBrush( YLogFileFigures::JITTER_BRUSH ) ),
zoomfactorx(YTIMING_DEFAULT_ZOOMFACTOR),
legendfactorx(YTIMING_DEFAULT_LEGENDFACTOR)
{}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// posy: left top position of timing widget
void YPainter::DrawLoggingAction(YLogAction *action, int posy)
{
	int				posx;
	int				type=action->GetAction();
	YASA_TIME		time=action->GetStartTime() < painterlogfileargs.cxtime ? painterlogfileargs.cxtime : action->GetStartTime();
	YASA_TIME		endtime=0;

	if ( type )
	{
		posx=painterlogfileargs.xroot+(time-painterlogfileargs.cxtime)/GetZoomFactor();
		posy=painterlogfileargs.yroot+posy;

		// draw task range if set ?
		switch ( action->GetAction() )
		{
		case YASAL_IDLE_THREAD:		if ( painterlogfileargs.drawtaskidlerange )
									{
										endtime= action->GetEndTime() < painterlogfileargs.crtime ? action->GetEndTime() : painterlogfileargs.crtime;
										painter->fillRect( posx								, posy+(YTIMING_TIMING_HEIGHT-YTIMING_TASK_HEIGHT),
														   (endtime-time)/GetZoomFactor()	, YTIMING_TASK_HEIGHT,
														   taskidlebrush);
									}
									break;
		case YASAL_SCHEDULER_CALLED:if ( painterlogfileargs.drawsystemrange )
									{
										endtime= action->GetEndTime() < painterlogfileargs.crtime ? action->GetEndTime() : painterlogfileargs.crtime;
										painter->fillRect( posx								, posy+(YTIMING_TIMING_HEIGHT-YTIMING_TASK_HEIGHT),
														   (endtime-time)/GetZoomFactor()	, YTIMING_TASK_HEIGHT,
														   systemtaskbrush);
									}
									break;
		case YASAL_THREAD_EXCLUDED:	if ( painterlogfileargs.drawtaskexcludedrange )
									{
										endtime= action->GetEndTime() < painterlogfileargs.crtime ? action->GetEndTime() : painterlogfileargs.crtime;
										painter->fillRect( posx								, posy+(YTIMING_TIMING_HEIGHT-YTIMING_TASK_HEIGHT),
														   (endtime-time)/GetZoomFactor()	, YTIMING_TASK_HEIGHT,
															taskexcludedbrush);
									}
									break;
		case YASAL_MUTEX_LOCKED:	if ( painterlogfileargs.drawmutexrange && action->IsMutexRange() )
									{
										endtime= action->GetEndTime() < painterlogfileargs.crtime ? action->GetEndTime() : painterlogfileargs.crtime;
										painter->fillRect( posx								, posy+(YTIMING_TIMING_HEIGHT-YTIMING_TASK_HEIGHT),
														   (endtime-time)/GetZoomFactor()	, YTIMING_TASK_HEIGHT,
														   mutexbrush);
									}
									break;
		case YASAL_JITTER:			if ( painterlogfileargs.drawjitterrange )			// show jitter
									{
										endtime= action->GetEndTime() < painterlogfileargs.crtime ? action->GetEndTime() : painterlogfileargs.crtime;
										//if ( time!=endtime )
										{
											painter->fillRect( posx								, posy+(YTIMING_TIMING_HEIGHT-YTIMING_TASK_HEIGHT),
															   (endtime-time)/GetZoomFactor()	, YTIMING_TASK_HEIGHT,
															   jitterbrush);
										}
									}

									break;
		default:					if ( painterlogfileargs.drawtaskrange && action->IsThreadRange() )
									{
										if ( painterlogfileargs.drawtaskrange )
											endtime= action->GetEndTime() < painterlogfileargs.crtime ? action->GetEndTime() : painterlogfileargs.crtime;
											painter->fillRect( posx								, posy+(YTIMING_TIMING_HEIGHT-YTIMING_TASK_HEIGHT),
															   (endtime-time)/GetZoomFactor()	, YTIMING_TASK_HEIGHT,
															   taskbrush);
									}
									break;
		}


		// draw logging action pixmaps
		const QPixmap	*pixmap=logfilepixmaps.GetLogFilePixmap(type);
		if (pixmap ) //&& time<=action->GetStartTime() )
		{
			posx=painterlogfileargs.xroot+(action->GetStartTime()-painterlogfileargs.cxtime)/GetZoomFactor();
			painter->drawPixmap( posx-( pixmap->width()+1)/2, posy, *pixmap );
		}


		// Print all figures. This is a test if all figures are available
		/*for ( unsigned int i=0; i<YLogFileFigures::MAX_LOGGINGFIGURES; i++)
		{
			pixmap=logfilepixmaps.GetLogFilePixmap(i);
			if (pixmap)
				painter->drawPixmap( i*20+10, 600, *pixmap );
		}*/
	}

}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Set a new zoom factor
/*
void YPainter::SetZoomFactor(int z)
{
	// calculate a default zoom factor
	if ( z<=0 )
	{
		z=YTIMING_DEFAULT_ZOOMFACTOR;
	}

	zoomfactorx=z;

	if ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR < legendfactorx/zoomfactorx )
	{
		while ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR < legendfactorx/zoomfactorx )
			legendfactorx/=YTIMING_LEGENDFACTOR_CHANGE;
	}
	else
	{
		if ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR > legendfactorx/zoomfactorx )
		{
			while ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR > legendfactorx/zoomfactorx )
			legendfactorx*=YTIMING_LEGENDFACTOR_CHANGE;
		}
	}

	// round legend factor to power of 10th
	// fast hack: for (int i=1; i< YTIME::YASA_TIME_INT_MAX; i=i*10)
	for (YASA_TIME i=1; i< YTIME::YASA_TIME_INT_MAX; i=i*10)
	{
		if (i>legendfactorx)
			break;
		legendfactorx=(legendfactorx+i/2)/i*i;
	}
}
*/

void YPainter::SetZoomFactor(YASA_TIME maxtime, int z)
{
	//int a;
	// calculate a default zoom factor
	if ( z<=0 )
	{
		z=YTIMING_DEFAULT_ZOOMFACTOR;
	}

	zoomfactorx=z;

	//a=(int)(maxtime/zoomfactorx);
	while ( maxtime/zoomfactorx > 0x7fffffff )
		zoomfactorx*=2;

	if ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR < legendfactorx/zoomfactorx )
	{
		while ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR < legendfactorx/zoomfactorx )
			legendfactorx/=YTIMING_LEGENDFACTOR_CHANGE;
	}
	else
	{
		if ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR > legendfactorx/zoomfactorx )
		{
			while ( YTIMING_DEFAULT_LEGENDFACTOR/YTIMING_DEFAULT_ZOOMFACTOR > legendfactorx/zoomfactorx )
			legendfactorx*=YTIMING_LEGENDFACTOR_CHANGE;
		}
	}

	// round legend factor to power of 10th
	// fast hack: for (int i=1; i< YTIME::YASA_TIME_INT_MAX; i=i*10)
	for (YASA_TIME i=1; i< YTIME::YASA_TIME_INT_MAX; i=i*10)
	{
		if (i>legendfactorx)
			break;
		legendfactorx=(legendfactorx+i/2)/i*i;
	}
}


