//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Timing diagram windows code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/timingwindows.cpp,v 1.3 2003/01/23 09:18:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "timingwindows.h"
#include "timinghandler.h"
#include "timings.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YFrame* YResultFrame::CreateTimingFrame(QWidget *parent, YFrameArgs &args)
{
	YTimingResults	*timingresult = (YTimingResults*) args.GetObject();
	// first parent points to YResult
	// second parent points to YProject or YEnvironment depending on YResult::type
	if ( timingresult && timingresult->GetParent() && timingresult->GetParent()->GetParent() )
	{
		switch ( timingresult->GetParent()->GetResultArgs().GetResultType() )
		{
		case YResultArgs::PROJECT_RESULTS:		return new YTimingProjectResultFrame(parent, args);
		case YResultArgs::ENVIRONMENT_RESULTS:	return new YTimingEnvironmentResultFrame(parent, args);
		default:								break;
		}
	}
	qWarning("YTimingFrame::CreateTimingFrame() failed!");
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YResultHandler* YTimingEnvironmentResultFrame::CreateResultHandler(QSplitter *parent) const
{
	return new YDiagramNameTimingHandler(parent);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTimingEnvironmentResultFrame::InsertResults(YResultHandler *resulthandler) const
{
	YEnvironment		*environment	= GetEnvironment();
	int					retvalue=YASAE_ERROR_IN_ARGUMENTS;;
	YTimingHandler		*timinghandler=(YTimingHandler*)resulthandler;

	if ( timinghandler && environment)
	{
		if ( environment->GetEnvironmentLogFileSet().GetTaskSet().GetCount() )
		{
			retvalue=timinghandler->InsertTiming( new YTaskEnvironmentTimings( environment->GetEnvironmentLogFileSet() ) );
			if (retvalue)
				return retvalue;
		}
		if ( environment->GetEnvironmentLogFileSet().GetMutexSet().GetCount() )
		{
			retvalue=timinghandler->InsertTiming( new YMutexEnvironmentTimings(environment->GetEnvironmentLogFileSet() ) );
			if (retvalue)
				return retvalue;
		}
		if ( environment->GetEnvironmentLogFileSet().GetCount() )
		{
			retvalue=timinghandler->InsertTiming( new YCPUEnvironmentTimings(  environment->GetEnvironmentLogFileSet() ) );
			if (retvalue)
				return retvalue;
		}
		retvalue=timinghandler->PostCreate();
	}
	return retvalue;
}










//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YResultHandler* YTimingProjectResultFrame::CreateResultHandler(QSplitter *parent) const
{
	return new YDiagramNameTimingHandler(parent);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTimingProjectResultFrame::InsertResults(YResultHandler *resulthandler) const
{
	YProject				*project= GetProject();
	YEnvironmentLogFileSet	*logfileset;
	YProjectTimings			*projecttiming;
	int						id,retvalue=YASAE_ERROR_IN_ARGUMENTS;;
	unsigned int			index, index2, index3;
	YTimingHandler			*timinghandler=(YTimingHandler*)resulthandler;

	if ( timinghandler )
	{
		// create task timings, parse each environment
		for ( index2=0; index2<project->GetEnvironmentSet().GetCount() ; index2++ )
		{
			// check tasks of the current environment
			logfileset=(YEnvironmentLogFileSet*) &((YEnvironment*)project->GetEnvironmentSet().At(index2) )->GetEnvironmentLogFileSet();
			for (index=0; index<logfileset->GetTaskSet().GetCount(); index++)
			{
				// is this task id already inserted ?
				id			= ( (YLogFileTask*) logfileset->GetTaskSet().At(index) )->GetId();
				for( index3=0; index3< timinghandler->GetTimingInfos()->count(); index3++)
				{
					projecttiming=(YProjectTimings*)timinghandler->GetTimingInfos()->at(index3)->GetTimings();
					if ( projecttiming->GetTimingTypeName() == MSGP_TI_SUBTITLE_TASK )
					{
						if ( ((YTaskProjectTimings*) projecttiming )->GetTaskID() == id )
							break;
					}
				}
				// if index3==count() -> task not found
				if ( index3 == timinghandler->GetTimingInfos()->count() )
				{
					retvalue=timinghandler->InsertTiming(
						new YTaskProjectTimings( ((YLogFileTask*) logfileset->GetTaskSet().At(index))->GetName() ,
												 ((YLogFileTask*) logfileset->GetTaskSet().At(index))->GetId(),
												 project->GetProjectLogFile() ) );
				}
			}
		}


		// create mutex timings, parse each environment
		for ( index2=0; index2<project->GetEnvironmentSet().GetCount() ; index2++ )
		{
			// check mutex' of the current environment
			logfileset=(YEnvironmentLogFileSet*) &((YEnvironment*)project->GetEnvironmentSet().At(index2) )->GetEnvironmentLogFileSet();
			for (index=0; index<logfileset->GetMutexSet().GetCount(); index++)
			{
				// is this mutex id already inserted ?
				id			= ( (YLogFileMutex*) logfileset->GetMutexSet().At(index) )->GetId();
				for( index3=0; index3< timinghandler->GetTimingInfos()->count(); index3++)
				{
					projecttiming=(YProjectTimings*)timinghandler->GetTimingInfos()->at(index3)->GetTimings();
					if ( projecttiming->GetTimingTypeName() == MSGP_TI_SUBTITLE_MUTEX )
					{
						if ( ((YMutexProjectTimings*) projecttiming )->GetMutexID() == id )
							break;
					}
				}
				// if index3==count() -> mutex not found
				if ( index3 == timinghandler->GetTimingInfos()->count() )
				{
					retvalue=timinghandler->InsertTiming(
						new YMutexProjectTimings( ((YLogFileMutex*) logfileset->GetMutexSet().At(index))->GetName() ,
												 ((YLogFileMutex*) logfileset->GetMutexSet().At(index))->GetId(),
												 project->GetProjectLogFile() ) );
				}
			}
		}

		// create cpu timings, parse each environment
		for ( index=0; index<project->GetEnvironmentSet().GetCount() ; index++ )
		{
			logfileset=&((YEnvironment*)project->GetEnvironmentSet().At(index))->GetEnvironmentLogFileSet();
			for ( index3=0; index3<logfileset->GetCount() ; index3++ )
			{
				// is this task id already inserted ?
				for( index2=0; index2< timinghandler->GetTimingInfos()->count(); index2++)
				{
					projecttiming=(YProjectTimings*) timinghandler->GetTimingInfos()->at(index2)->GetTimings();
					if ( projecttiming->GetTimingTypeName() == MSGP_TI_SUBTITLE_CPU )
					{
						if ( ((YCPUProjectTimings*) projecttiming )->GetCPU() == index3 )
							break;
					}
				}
				// if index2 != count() -> cpu not found
				if ( index2 == timinghandler->GetTimingInfos()->count() )
				{
					retvalue=timinghandler->InsertTiming(
						new YCPUProjectTimings( index3, project->GetProjectLogFile() ) );
				}
			}
		}

		retvalue=timinghandler->PostCreate();
	}
	return retvalue;
}







