//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA work space manager
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/workspacemanager.cpp,v 1.4 2003/01/23 09:18:40 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qmessagebox.h>
#include <qfileinfo.h>
#include <qdir.h>
#include "workspacemanager.h"
#include "workspace.h"
#include "parser.h"
#include "application.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YWorkSpaceManager::YWorkSpaceManager(YApplication &app) :
application(app),
workspace(0),
filedialog( app.GetStartPath(), MSGP_CHOOSE_WORKSPACE_FILE, YASA_WORKSPACE_FILTER)
{}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int	YWorkSpaceManager::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_WORKSPACE) )
	{
		if ( workspace )
		{
			return parser.NewSection(YPC_WORKSPACE, *workspace );
		}
		else
			return YASAE_INTERNAL_ERROR;
	}
	return parser.SectionEnd();							// no name, description or id
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int	YWorkSpaceManager::GetConfig(YOutputParser &parser)
{
	int retvalue;
	retvalue=parser.NewSection(YPC_WORKSPACE, *workspace);
		if ( retvalue )	return retvalue;
	return parser.SectionEnd();							// no name, description or id
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Open a already saved project
void YWorkSpaceManager::OpenWorkSpace()
{
	int			retvalue;
	QString		filename;

	if ( workspace )
	{
		QFileInfo	fi( workspace->GetAbsFilePath() );
		retvalue=filedialog.Request( fi );
	}
	else
		retvalue=filedialog.Request( QFileInfo( filedialog.selectedFile() ) );
	if ( ! retvalue  )
		ReadWorkSpace(filedialog.selectedFile());					// open project
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Save workspace under a new name
void YWorkSpaceManager::SaveWorkSpace()
{
	if ( workspace )
	{
		QFileInfo	fi( QDir(workspace->GetAbsFilePath()), workspace->GetFileName() );
		if ( fi.filePath().length() && !(fi.isDir() ) )
			WriteWorkSpace( fi.filePath() );
		else
			SaveWorkSpaceAs();
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Save workspace under a new name
void YWorkSpaceManager::SaveWorkSpaceAs()
{
	int			retvalue;
	QString		filename;

	if ( workspace )
	{
		//retvalue=filedialog.Request( workspace->GetFileInfo(), QFileDialog::AnyFile );
		retvalue=filedialog.Request( workspace->GetAbsFilePath(), QFileDialog::AnyFile );
		if ( ! retvalue  )
		{
			// need extension or already set ?
			QFileInfo finfo(filedialog.selectedFile());
			filename = filedialog.selectedFile();
			if ( ! finfo.extension().length() )
				filename+=YASA_POSTFIX;

			// overwrite file ?
			QFileInfo	finfoext(filename);
			//if ( workspace->GetFileInfo().absFilePath() != finfoext.absFilePath()  )
			if ( workspace->GetAbsFilePath() != finfoext.absFilePath()  )
			{
				if ( finfoext.exists() )
				{
					QString	text=MSGP_OVERWRITE_QUESTION;
					text=text.arg(filename);
					if ( QMessageBox::warning( 0, MSGP_WARNING, text, MSGP_ERROR_YES,
						 MSGP_ERROR_NO, 0, 0, 1 ) != 0 )
						return;		// YASAE_ENTRY_ALREADY_EXISTS;
				}
			}
			WriteWorkSpace(filename);
		}
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YWorkSpaceManager::CreateNewWorkSpace()
{
	if ( workspace )
	{
		//if ( workspace->GetChanged() )
		{
			if ( QMessageBox::warning( 0, MSGP_WARNING, MSGP_NEWPROJECT_QUESTION, MSGP_ERROR_YES,
				 MSGP_ERROR_NO, 0, 0, 1 ) != 0 )
				return YASAE_OPERATION_ABORTED;
		}
		CloseWorkSpace();
	}
	workspace=new YWorkSpace(application);
	if (!workspace)
		return YASAE_OUT_OF_MEMORY;
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YWorkSpaceManager::NewWorkSpace()
{
	int retvalue;
	retvalue = CreateNewWorkSpace();
	if ( ! retvalue )
	{
		emit NewWorkSpaceCreated(workspace);
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YWorkSpaceManager::ReadWorkSpace(const QString &filename)
{
	int						retvalue;
	YXMLInputParser			parser(YPC_YASA_WORKSPACE);

	retvalue=CreateNewWorkSpace();
	if ( ! retvalue )
	{
		workspace->SetFilePath(filename);
		retvalue=parser.Parse(filename, *this);
		if ( ! retvalue )
		{
			emit NewWorkSpaceCreated(workspace);
			workspace->SetChanged(false);
		}
		else
			CloseWorkSpace();
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YWorkSpaceManager::WriteWorkSpace(const QString &newfilename)
{
	YXMLOutputParser		parser;

	if ( workspace )
	{
		workspace->SetFilePath(newfilename);
		return parser.Parse(*this, newfilename );
	}
	return YASAE_INTERNAL_ERROR;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpaceManager::CloseWorkSpace()
{
	if ( workspace )
	{
		filedialog.setSelection( workspace->GetAbsFilePath() );					// store old file path for file dialog
		YWorkSpace *tmp=workspace;
		emit WorkSpaceDestroyed(workspace);
		workspace=0;
		delete tmp;
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YWorkSpace* YWorkSpaceManager::GetWorkSpace()
{
	if ( this )
		return workspace;
	else
		qWarning("YWorkSpaceManager::GetWorkSpace() called but this is zero!");
	return 0;
}