//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA work space tree view definitions
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/workspacetreeview.cpp,v 1.3 2003/01/23 09:18:40 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "workspacetreeview.h"
#include "translations.h"
#include "actions.h"
#include "actionhandler.h"
#include "application.h"
#include "settings.h"


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Create project tree dependig the current project. The project must be initialized correctly!
int YWorkSpaceTreeView::InitInstance()
{
	YApplicationFigures::SetListViewBackdrop(*this, YApplicationFigures::BACKGROUND_WORKSPACE_TREE );

/*	popupmenu = new QPopupMenu( this , MSGP_YASA );
	if ( ! popupmenu )
		return YASAE_OUT_OF_MEMORY;
*/

	connect( this, SIGNAL (	mouseButtonPressed(int, QListViewItem *, const QPoint &, int ) ),
			 this, SLOT (	MouseButtonClicked(int, QListViewItem *, const QPoint &,int ) ) );
/*	Attention: RightButtonPressed will be called twice if you uncomment these lines -> GURU
	connect( this, SIGNAL (	contextMenuRequested(QListViewItem *, const QPoint &, int ) ),
			 this, SLOT (	RightButtonClicked(QListViewItem *, const QPoint &, int ) ) );
*/	connect( this, SIGNAL (	doubleClicked(QListViewItem *) ),
			 this, SLOT (	DoubleClicked(QListViewItem *) ) );
	connect( this, SIGNAL (	returnPressed(QListViewItem * ) ),
			 this, SLOT (	ItemActivated(QListViewItem * ) ) );
#if QT_VERSION>=300
	connect( this, SIGNAL (	spacePressed(QListViewItem * ) ),
			 this, SLOT (	ItemActivated(QListViewItem * ) ) );
#endif
	return YListView::InitInstance();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpaceTreeView::MouseButtonClicked( int button, QListViewItem *item, const QPoint &point, int column)
{
	if ( item )
	{
		switch ( button )
		{
		case 1:	ItemActivated(item);
				break;
		case 2:	RightButtonClicked(item, point, column);
				break;
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpaceTreeView::ItemActivated( QListViewItem *item)
{
	if ( item )
	{
		YActionHandler::SetCurrentObject( ((YListViewItem*)item)->GetObject() );
		((YListViewItem*)item)->Activate();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpaceTreeView::DoubleClicked(QListViewItem *i)
{
	YListViewItem	*item;
	item=(YListViewItem*)i;
	if ( item->GetObject() && item->GetObject()->GetActions() )
	{
		int retvalue=item->GetObject()->GetActions() -> Activate(item->GetObject());
		if ( retvalue!=YASAE_NOT_IMPLEMENTED)
			item->setOpen( ! item->isOpen() );
	}
	else
		qWarning("YWorkSpaceTreeView::DoubleClicked() called but no object or actions found!");
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpaceTreeView::RightButtonClicked( QListViewItem *item, const QPoint &point, int b)
{
//	qWarning("YAction::RightButtonClicked() called! b=%d", b );
	if ( item )
	{
		YObject		*object=((YListViewItem*)item)->GetObject();
//		qWarning("YAction::RightButtonClicked() called! object:%s, ptr:0x%x, b=%d", object->GetName().latin1(), object, b );

		QPopupMenu *popupmenu = new QPopupMenu( this , MSGP_YASA );
		popupmenu->clear();

		if ( ! YActions::CreateMenu(object, popupmenu) )
		{
			YActionHandler::SetCurrentObject(object);
			popupmenu->move( point );
			popupmenu->exec();
		}
	}
	Q_UNUSED(b);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpaceTreeView::keyPressEvent(QKeyEvent * e)
{
	QString	k;
	k=e->text();
//	if ( k==a )
	{
		YListViewItem	*item;
		item=(YListViewItem*)currentItem();
		if (item)
		{
			if ( k=="a" )
			{
				//YActionHandler::GetActions(YActionHandler::PROJECTSET_ACTIONS)->Add(item->GetObject());
				return;
			}
			if ( k=="d" )
			{
				//YActionHandler::GetActions(YActionHandler::PROJECTSET_ACTIONS)->RemoveAll(item->GetObject());
				return;
			}
		}
	}
	e->ignore();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Clear the listview
void YWorkSpaceTreeView::ClearListView(YWorkSpace *workspace)
{
	clear();													// remove all items from list
	if ( columns() )
	{
		removeColumn(0);
		disconnect( workspace, SIGNAL(NewItem(YObject *)), this, SLOT(NewChildObject(YObject *) ) );
		disconnect( workspace, SIGNAL(ItemRemoved(YObject *)), this, SLOT(ChildObjectRemoved(YObject *) ) );
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Update the whole workspace tree
void YWorkSpaceTreeView::UpdateListView(YWorkSpace *workspace)
{
	YListViewItem	*item=GetCurrentItem();

	if ( item )
	{
		oldselected=item->GetObject();
	}
	else
		oldselected=0;
	ClearListView(workspace);
	if ( workspace )
	{
		YWorkSpaceTreeOutputParser	parser(*this, *workspace);

		//QSize t=this->size();									// does not work due to automatically width
		//setColumnWidth( 0, t.width() );
		addColumn( MSGP_CAPTION_WORKSPACE );
		setShowSortIndicator(FALSE);
		setSorting(-1);
		//setShowSortIndicator(TRUE);
		//setSorting(0,true);
		//setRootIsDecorated(true);
		//YActionHandler	*ah=YApplication::GetActionHandler();
		connect( workspace, SIGNAL(NewItem(YObject *)), this, SLOT(NewChildObject(YObject *) ) );
		connect( workspace, SIGNAL(ItemRemoved(YObject *)), this, SLOT(ChildObjectRemoved(YObject *) ) );
		parser.Parse();
		oldselected=0;
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new child object must be inserted
//! The child can be any derived type of YObject
void YWorkSpaceTreeView::NewChildObject(YObject *object)
{
	YWorkSpace *ws=YApplication::GetWorkSpaceManager()->GetWorkSpace();
	if (ws)
		UpdateListView(ws);
	Q_UNUSED(object);
}
//! The associated item must be removed from list
void YWorkSpaceTreeView::ChildObjectRemoved(YObject *object)
{
	YListViewItem *item=FindItemFromObject(object);
	if (item)
		delete item;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YListViewItem*	YWorkSpaceTreeView::CreateTreeItem(YObject &object, QListViewItem *parent, QListViewItem *after)
{
	YListViewItem	*item;

	if ( parent )
		item=new YListViewItem( &object, parent, after );
	else
	{	// This is the first item in our list -> projectset
		// we want that it behave like a workspace entry
		//object=((YSet*)object)->GetParent();
		item=new YListViewItem( &object, this );
	}
	if ( item )
	{
		YActions::CreateTreeItems( &object, parent, item );
		if ( oldselected && (&object) == oldselected )
			setSelected(item, true);
		return item;
	}
	qWarning("YWorkSpaceTree::CreateTreeItem() failed!\n");
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//int YWorkSpaceTreeOutputParser::Parse(QListView &parent, YWorkSpace &workspace)
int YWorkSpaceTreeOutputParser::Parse()
{
	YProjectSet		&set=workspace.GetProjectSet();
	return NewSectionSet(set.GetSectionKey(), set);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//int YWorkSpaceTreeOutputParser::NewSectionSet(CONFIG_STRING newsectionsetkey, YSet &yset, QListViewItem *parent )
int YWorkSpaceTreeOutputParser::NewSectionSet(CONFIG_STRING newsectionsetkey, YSet &yset)
{
	YObject			*entry;
	int				retvalue=0;
	unsigned int	index=0;
	YListViewItem	*item=0;
	QListViewItem	*oldafter=0;

	// do not insert tree items from resume and resource times
	if ( QString::compare(newsectionsetkey,YPC_RESUME_TIME)==0 ||
		QString::compare(newsectionsetkey,YPC_RESOURCE_TIME)==0 ||
		QString::compare(newsectionsetkey,YPC_SCHEDULER)==0)
		return 0;

	// create folder item, the "projects item was already created
	if ( count() )
		item = workspaceview.CreateTreeItem( yset, at( count()-1), afterlistview);
	else
		item = workspaceview.CreateTreeItem( yset, 0, 0 );

	if ( ! item )
		return YASAE_OUT_OF_MEMORY;
	retvalue=item->InitInstance();
	if ( ! retvalue )
	{
		insert( count(), item);
		item->setOpen(true);

		item=0;
		while ( !retvalue && index<yset.GetCount() )
		{
			entry=yset.At(index);
			if ( entry )
			{
				item = workspaceview.CreateTreeItem( *entry, at( count()-1), item );
				if ( item )
				{
					insert( count(), item );
					oldafter=afterlistview;
					retvalue=entry->GetConfig(*this);
					afterlistview=oldafter;
					item->setOpen(true);
					remove( count()-1 );
				}
			}
			index++;
		}
		afterlistview=at(count()-1);
		remove( count()-1 );
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int	YWorkSpaceTreeOutputParser::NewSection(const CONFIG_STRING newsectionkey, YObject &yobject )
{
	YListViewItem	*item;
	int	retvalue=YASAE_OUT_OF_MEMORY;

	//qWarning("YWorkSpaceTreeOutputParser::NewSection() should not be called! Key: %s, YObject: %s\n", newsectionkey, yobject.GetName().latin1() );

	if ( count() )
		item = workspaceview.CreateTreeItem( yobject, at( count()-1), afterlistview );
	else
		item = workspaceview.CreateTreeItem( yobject, 0, 0 );
	if ( item )
	{
		insert( count(), item );
		item->setOpen(true);
		retvalue=yobject.GetConfig(*this);
		afterlistview=at(count()-1);
		remove( count()-1 );
	}
	return retvalue;
	Q_CONSTUNUSED( newsectionkey );
}
